
PAGE  80,132

; BOOTC331.ASM  Boot sector - COMPAQ DOS 3.31
;
; 7.12.91       Disassembly from floppy boot sector 512-byte image (lsz)
;               Assembles error-free (MASM 5.0)
;               Byte-identical to Compaq DOS 3.31 floppy boot sector
;               Not tested

; ------------------- program notes -------------------------------------
comment ~

This is a disassembly of the boot sector of a Compaq DOS 3.31 diskette.

Program control is transferred to this code by the system BIOS
after it completes its POST routines.

This code loads ibmbio.com and then chains to it.

To create a boot sector image from this .asm file, first go
through the usual exercise of creating an .exe file, where
ww.asm is this working source file:

       C:\>masm ww,,ww;
       C:\>link ww,ww;
       C:\>exe2bin ww.exe ww.bin

The boot sector code is loaded by the system BIOS at 07c0:0000
(0000:7c00). This .asm file will generate a load image that has
the proper offsets and references at 7c0h.

When the ww.bin file created above is compared with an image
of the diskette boot sector, it will be byte-for-byte identical
except for the machine- and disk-format specific data bytes.

 R. Zider
 San Francisco

~tnemmoc


; ----- Data segment ---------------------------------------------------------
;
;       This section is to establish addressability of locations.
;       No code gets generated by the assembler.
;
dseg            segment at 0000
                org     1eh*4                   ; int 1Eh is ptr to disk parms
int_1Eh         dd      ?                       ;
                org     0472h
bios_resetflag  dw      ?                       ; (0000:0472) warm/cold boot flag
                org     500h
bioscommarea    db      ?                       ; BIOS communications area
                org     7c00h
boot_start      db      ?                       ; load point for this boot code
                                                ; alternate to 07c0:0000

; ----- labels for working storage locs that overlay initial code area

; ----- diskette parameter table                  ; 11-byte disk parms table
;
                org     7c2ah
disk_parms      db      ?               ; 1st specify byte
                db      ?               ; 2nd specify byte
                db      ?               ; motor wait time
                db      ?               ; 512 bytes per sector
                db      ?               ; EOT (last sector on track) (=0Fh?)
                db      ?               ; gap length
                db      ?               ; DTL
                db      ?               ; gap length for format
                db      ?               ; fill byte for format
                db      ?               ; head settle time (ms) <== machine dependent **
                db      ?               ; motor start time (1/8 secs)
dseg            ends

dseg2           segment at 70h
                org     0
ibmbio_loadpt   db      ?               ; (0070:0000) ibmbio gets loaded here
dseg2           ends

cseg            segment at 07c0h        ; segment value of boot code load point
cseg            ends

cseg2           segment at 0ffffh
                org     0
BIOS_reboot     db      ?                ; entry for reboot jump
cseg2           ends

; ------------- Code segment ---------------------------------------------
;
seg_a           segment byte public
                assume  cs:seg_a, ds:seg_a


                org     0                       ; loads at 07c0:0000

scpq331         proc    far

start:
                jmp     short loc_1             ; (002A)
                db      90h

; -------------- data section --------------------------------------------
;
oem_ver         db      'IBM  3.3'
bytes_per_sec   dw      200h                    ; 512 bytes/sector
secs_per_clust  db      2
reserv_secs     dw      1
fat_copies      db      2
dir_entries     dw      70h                     ; 112 directory entries
secs_on_disk    dw      2D0h                    ; 720 disk sectors (=360KB)
format_id       db      0FDh
secs_per_FAT    dw      2
secs_per_trk    dw      9
sides           dw      2
hidden_secs_lsw dw      0
hidden_secs_msw dw      0
                db      0, 0, 0, 0
ptr_sec_lsw     dw      0
ptr_sec_msw     dw      0
                db      0, 0


; -------------- code section
;
loc_1:

        cli                             ; Disable interrupts while stack setup
        xor     bp,bp                   ; Zero register
        mov     ax,cseg                 ; load point  07c0:0000
        mov     ds,ax                   ; set DS to 07c0h

; ----- pick up hidden sectors if this is hard drive
;
        les     bx,dword ptr hidden_secs_lsw    ; (0000:001C=0) Load 32 bit ptr
        mov     drive,dl                ; save drive (passed from BIOS boot code?)
        or      dl,dl                   ; Zero ?
        jns     loc_2                   ; Jump if not sign (<80h)
        mov     ptr_sec_lsw,bx          ; >=80h, so hard drive..
        mov     ptr_sec_msw,es          ;  ..pick up hidden sectors
loc_2:
        mov     es,bp                   ; ES to 0000

; ----- set up local stack from load point down
;
        mov     ss,bp                   ; setting as 0000:7C00
        mov     sp, offset boot_start   ; same as start at 07C0:0000

; ----- load disk parameter table and set new int 1Eh
;
        cld                             ; Clear direction
        push    ds                      ; temp save ds
        lds     si,dword ptr ss:int_1Eh ; (0000:0078=522h) Load 32 bit ptr
        mov     di,offset disk_parms    ; (0000:7C2A) overlays foregoing code
        mov     cx,0Bh                  ; copy 11-byte parm block
        rep     movsb                   ; Rep when cx >0 Mov [si] to es:[di]
        pop     ds                      ; restore ds
        mov     byte ptr ds:[2Eh],0Fh   ; (07C0:002E) overlay EOT with 15 as max sector
        mov     di,offset int_1Eh       ; (0000:0078=22h)
        mov     ax, offset disk_parms
        stosw                           ; Store ax to es:[di] set new int 1Eh ptr
        xchg    ax,cx                   ; cx is zero
        stosw                           ; Store ax to es:[di] set seg=0
        sti                             ; Enable interrupts

; ----- reset boot disk (note: no failure code -- ignores results)
;
        mov     dl,drive                ; (07C0:01FD=0)
        int     13h                     ; Disk  dl=drive a  ah=func 00h
                                        ;  reset disk, al=return status

; ----- calc starting logical sector for directory
;
        mov     al,fat_copies           ; (07C0:0010=2)
        cbw                             ; Convrt byte to word
        mul     secs_per_FAT            ; (07C0:0016=2) ax = data * ax
        add     ax,reserv_secs          ; (07C0:000E=1)
        call    add_ax_bp               ; (00F4) bump next sector pointer

; ----- get set to read directory into buffer (BIOS communications area)
;
        call    calc_C_H_N              ; (00FD) get physical location
        mov     bx,offset bioscommarea  ; (0000:0500=0) load buffer for directory
        push    bx                      ; save ptr to head of buffer
        call    read_sec                ; (012B) read in the directory

; ----- compare first 11-byte directory entry against 'IBMBIO  COM'
;
        pop     di                      ; di has head of buffer
        mov     si,offset ibmbio_str    ; (07C0:0171=49h) 'IBMBIO  COM'
        mov     cx,0Bh                  ; check first 11-byte dir entry
                                        ;  (should be 'IBMBIO  COM')
        nop
        repe    cmpsb                   ; Rep zf=1+cx >0 Cmp [si] to es:[di]
        jnz     Nonsys_reboot           ; Jump if not zero

; ----- compare second directory entry against 'IBMDOS  COM'
;
        add     di,15h                  ; bump ptr to next dir entry
        mov     cl,0Bh                  ; check second 11-byte dir entry
                                        ;  ('IBMDOS  COM')
        nop
        nop
        repe    cmpsb                   ; Rep zf=1+cx >0 Cmp [si] to es:[di]
        jnz     Nonsys_reboot           ; Jump if not zero

; ----- First two directory names ok, so proceed to load ibmbio.com
;
        mov     ax,es:[bx+1Ch]          ; length if ibmbio.com in dir entry
        cwd                             ; Word to double word
        mov     cx,bytes_per_sec        ; (07C0:000B=200h)
        add     ax,cx
        dec     ax
        div     cx                      ; ibmbio.com length in sectors
        cmp     ax,14h                  ; if more than 20 sectors?
        jg      loc_3                   ; Jump if >
        mov     al,14h                  ; take only first 20 sectors
                                        ; (512*20=10240 bytes buffer limit?)
loc_3:
        xchg    ax,si                   ; si will now have number of
                                        ;  sectors to read (ibmbio.com)
        mov     ax,dir_entries          ; (07C0:0011=70h)
        mov     cl,4                    ; 16 32-byte directory entries/sector
        shr     ax,cl                   ; Shift w/zeros fill
        call    add_ax_bp               ; now points to first sector in
                                        ;  data area (ibmbio.com)
        push    ptr_sec_lsw             ; (07C0:0024) save offset
        les     bx,dword ptr p_ibmbioload       ; (07C0:016D=0) Load 32 bit ptr

get_next_sector:
        call    calc_C_H_N              ; (00FD)
        call    read_sec                ; (012B)
        sub     si,ax                   ; reduce by sectors read
        jbe     chain_to_ibmbio         ; Jump if below or =
        call    add_ax_bp               ; bump next sector pointer by
                                        ; number of sectors read
        push    dx                      ; temp save
        mul     bytes_per_sec           ; (07C0:000B=200h) ax = data * ax
        add     bx,ax                   ; bx has bytes offset into es:bx
                                        ;  read buffer
        pop     dx                      ; restore
        jmp     short get_next_sector   ; (00CA)

; ----- ibmbio.com now loaded, so transfer control to it at 0070:0000
;
chain_to_ibmbio:
        pop     bx                      ; restore ptr to beginning of es:bx
                                        ;  read buffer
        mov     ch,format_id            ; (07C0:0015=0FDh)
        mov     dl,drive                ; (07C0:01FD=0)
        jmp     dword ptr p_ibmbioload  ; (07C0:016D=0)

Nonsys_reboot:
        mov     si,offset Nonsys_msg    ; (07C0:018B=0Ah)
        jmp     short Replace_reboot    ; (0147)
        db      90h

scpq331         endp

;
;         accumulate logical sector pointer
;

add_ax_bp       proc    near
        add     ptr_sec_lsw,ax          ; (07C0:0024=0)
        adc     ptr_sec_msw,bp          ; (07C0:0026=0)
        retn
add_ax_bp       endp


;
;         calculate C-H-N physical location
;

calc_C_H_N      proc    near
        mov     ax,secs_per_trk         ; (07C0:0018=9)
        mul     byte ptr sides          ; (07C0:001A=2) ax = data * al
        xchg    ax,cx
        mov     ax,ptr_sec_lsw          ; (07C0:0024=0)
        mov     dx,ptr_sec_msw          ; (07C0:0026=0)
        div     cx                      ; ax,dx rem=dx:ax/reg
        xchg    ax,dx
        mov     cx,secs_per_trk         ; (07C0:0018=9)
        div     cl                      ; al, ah rem = ax/reg
        sub     cl,ah
        xchg    ax,cx
        inc     ch
        xchg    ch,cl
        ror     dh,1                    ; Rotate
        ror     dh,1                    ; Rotate
        or      dh,cl
        xchg    dh,dl
        xchg    cx,dx
        mov     dl,drive                ; (07C0:01FD=0)
        retn
calc_C_H_N      endp


;
;         read a single sector
;

read_sec        proc    near
        mov     di,5                    ; retry 5 times
loc_4:
        mov     ax,201h                 ; al=1: read one sector
        int     13h                     ; Disk  dl=drive a  ah=func 02h
                                        ;  read sectors to memory es:bx
        jc      loc_6                   ; Jump if carry Set
loc_5:
        mov     al,1
        retn
loc_6:
        cmp     ah,11h                  ; ah=11 disk recalibrate function
        je      loc_5                   ; Jump if equal
        xor     ax,ax                   ; Zero register
        int     13h                     ; Disk  dl=drive a  ah=func 00h
                                        ;  reset disk, al=return status
        dec     di
        jnz     loc_4                   ; Jump if not zero
        mov     si,offset DBfail_msg    ; (07C0:01D5=0Ah)

Replace_reboot:
        call    tty_msg                 ; (0167)
        mov     si,offset Replace_msg   ; (07C0:01AB=0Ah)
        call    tty_msg                 ; (0167)
        xor     ax,ax                   ; Zero register
        int     16h                     ; Keyboard i/o  ah=function 00h
                                        ;  get keybd char in al, ah=scan
        mov     word ptr ss:bios_resetflag,1234h        ; (0000:0472=1200h)
        jmp     far ptr BIOS_reboot     ;*(FFFF:0000)
read_sec        endp


;
;         byte in al to console
;

tty_al          proc    near
loc_7:
        mov     ah,0Eh
        mov     bx,7
        int     10h                     ; Video display   ah=functn 0Eh
                                        ;  write char al, teletype mode

; External Entry into Subroutine 
;
;         $-terminated string to console

tty_msg:
        lodsb                           ; String [si] to al
        cmp     al,24h                  ; '$'
        jne     loc_7                   ; Jump if not equal
        retn
tty_al          endp


p_ibmbioload    dd      far ptr ibmbio_loadpt
ibmbio_str      db      'IBMBIO  COM'
ibmdos_str      db      'IBMDOS  COM'
                db      0, 0, 0, 0
Nonsys_msg      db      0Ah, 0Dh, 'Non-System disk or dis'
                db      'k error$'
Replace_msg     db      0Ah, 0Dh, 'Replace and strike any'
                db      ' key when ready', 0Ah, 0Dh, '$'
DBfail_msg      db      0Ah, 0Dh, 'Disk boot failure$'
cpq_copyright   db      'Copr. COMPAQ 1983-87'
drive           db      0
                db       55h,0AAh

seg_a           ends



        end     start
