
PAGE  80,132
; BOOTi300.ASM  Boot sector - IBM PC DOS 3.00
;
; 7.11.91       Disassembly from floppy boot sector 512-byte image (lsz)
;               Assembles error-free (MASM 5.0)
;               Byte-identical to PC DOS 3.0 floppy boot sector
;               Not tested

; ------------------- program notes -------------------------------------
comment ~

This is a disassembly of the boot sector of an IBM PC DOS 3.00 diskette.

Program control is transferred to this code by the system BIOS
after it completes its POST routines.

This code loads io.sys and then chains to it.

To create a boot sector image from this .asm file, first go
through the usual exercise of creating an .exe file, where
ww.asm is this working source file:

       C:\>masm ww,,ww;
       C:\>link ww,ww;

The boot sector code is loaded by the system BIOS at 0000:7c00.
This .asm file will generate a load image that has the proper
offsets and references at 7c00h.  However, since MASM does not
generate by itself binary images and exe2bin does not convert
to binary  modules .exe files that have been org'ed to anything
other than 100h, the 512-byte binary image has to be manually
extracted from the full segment image produced by the linker
(7c00h+200h=7e00h or 32256 bytes).

To generate only the 512-byte sector image, run a small .bat
file with the following code:

     echo nww.bin      >cxe2bin.arf
     echo rcx          >>cxe2bin.arf
     echo 200          >>cxe2bin.arf
     echo w7c00        >>cxe2bin.arf
     echo q            >>cxe2bin.arf
     debug ww.exe      <cxe2bin.arf

When the ww.bin file created above is compared with an image
of the diskette boot sector, it will be byte-for-byte identical
except for the machine- and disk-format specific data bytes.

 R. Zider
 San Francisco

~tnemmoc
; ===========================================================================



; ----- Data segment ---------------------------------------------------------
;
;       This section is to establish addressability of locations.
;       No code gets generated by the assembler.
;
dseg1           segment at 0000h
                org     1eh*4
int_1Eh         dd      ?                       ; Int 1Eh: ptr to diskette parms
dseg1           ends

dseg2           segment at 0050h
                org     0
bioscommarea    db      ?                       ; BIOS communications area
                org     1Ch
ibmbio_len      dw      ?                       ; file length within dir entry
                org     20h                     ; 2nd 32-bit dir entry
dir_2nd_entry   dw      ?                       ; (0050:0020)
dseg2           ends

dseg3           segment at 0070h
                org     0h
ibmbio_loadpt   db      ?                       ; (0000:0700)
dseg3           ends

dseg4           segment at 0000h
; ----- labels for working storage locs that overlay initial code area
                org     7c2ch
directory_sec   dw      ?                       ; logical sec # of directory
sec_to_get      dw      ?                       ; logical sector number
C_track         dw      ?                       ; cylinder in C-H-N
dseg4           ends


; ------------- Code segment ---------------------------------------------
;
seg_a           segment byte public
                assume  cs:seg_a, ds:seg_a


                org      7c00h                  ; loaded at 0000:7c00 by bootstrap loader

booti300        proc     far

start:
                jmp     short loc_1             ; (7C2C)
                db      90h

; -------------- data section --------------------------------------------
;
oem_ver         db      'IBM  3.0'
bytes_per_sec   dw      200h
secs_per_clust  db      2
reserv_secs     dw      1
fat_copies      db      2
dir_entries     dw      70h                     ; 112 directory entries
secs_on_disk    dw      2D0h                    ; 720 sectors
format_id       db      0FDh                    ; DSDD 5.25" 360KB diskette
N_sector        label   byte
secs_per_FAT    dw      2
secs_per_trk    dw      9
sides           dw      2
hidden_secs     dw      0
drive           db      0
H_head          db      0
secs_ibmbio     db      14h
; disk_parms      db       00h, 00h, 00h, 00h, 0Fh, 00h
;                 db       00h, 00h, 00h, 00h, 00h

; ----- diskette parameter table
;
; Below are typical parameters that were used in DOS 2.0 disk format.
;   In DOS 3.0, most of these parameters are no longer specified in
;   the diskette boot sector (bytes are zero).  Instead, these bytes
;   are now loaded from the BIOS parameter block pointed to by int 1Eh.
;   Only the EOT byte is disk media dependent, which specifies a maximum
;   number of sectors (here, 0Fh or 15).
;
; disk_parms      db      0dfh            ; 1st specify byte
;                 db      2               ; 2nd specify byte
;                 db      25h             ; motor wait time
;                 db      2               ; 512 bytes per sector
;                 db      9               ; EOT (last sector on track)
;                 db      02ah            ; gap length
;                 db      0ffh            ; DTL
;                 db      050h            ; gap length for format
;                 db      0f6h            ; fill byte for format
;                 db      0fh             ; H_head settle time (ms) <== machine dependent **
;                 db      02              ; motor start time (1/8 secs)

disk_parms      db      00              ; 1st specify byte
        db      00              ; 2nd specify byte
        db      00              ; motor wait time
        db      00              ; 512 bytes per sector
        db      0Fh             ; EOT (last sector on track)
        db      00              ; gap length
        db      00              ; DTL
        db      00              ; gap length for format
        db      00              ; fill byte for format
        db      00              ; H_head settle time (ms) <== machine dependent **
        db      00              ; motor start time (1/8 secs)


; -------------- code section
;
loc_1:
; ----- set up local stack from load point down
;
        cli                             ; Disable interrupts while stack setup
        xor     ax,ax                   ; Zero register
        mov     ss,ax                   ; SS to 0000
        mov     sp,offset start         ; stack below start of program

; ----- load disk parameter table and set new int 1Eh
;
        mov     bx,offset int_1Eh       ; BIOS disk parameters ptr int 1Eh
        lds     si,dword ptr ss:[bx]    ; Load 32 bit ptr
        push    ds                      ; save old ptr on stack
        push    si
        push    ss                      ; save ptr to new parms as well
        push    bx
        mov     es,ax                   ; ES to 0000
        mov     di,offset disk_parms    ; (0000:7C21) table to be loaded
        mov     cx,0Bh                  ; copy 11-byte parm table
        cld                             ; Clear direction

locloop_2:
        lodsb                           ; String [si] to al
        cmp     byte ptr es:[di],0      ; don't overlay non-zero values
        je      loc_3                   ; Jump if equal
        mov     al,es:[di]              ; use old value if non-zero
loc_3:
        stosb                           ; Store al to es:[di]
        loop    locloop_2               ; Loop if cx > 0

        mov     ax,es                   ; DS to ES (0000)
        mov     ds,ax
        mov     [bx+2],ax               ; new int 1Eh segment
        mov     word ptr [bx],offset disk_parms ; new int 1Eh offset
        sti                             ; Enable interrupts

; ----- reset boot diskette drive a:
;
        int     13h                     ; Disk  dl=drive a  ah=func 00h
                                        ;  reset disk, al=return status
        jnc     loc_4                   ; Jump if carry=0
        jmp     DBfail_freeze           ; (7D2E)

; ----- calc starting logical sector for directory
;
loc_4:
        push    cs                      ; DS to CS
        pop     ds
        mov     al,fat_copies           ; (0000:7C10=2)
        cbw                             ; Convrt byte to word
        mul     secs_per_FAT            ; (0000:7C16=2) ax = data * ax
        add     ax,hidden_secs          ; (0000:7C1C=0)
        add     ax,reserv_secs          ; (0000:7C0E=1)
        mov     directory_sec,ax        ; (0000:7C2C) ptr to start of dir
        mov     sec_to_get,ax           ; (0000:7C2E) cumulate

; ----- now calc starting logical sector of data area
;
        mov     ax,20h                  ; 32-bit directory entries
        mul     dir_entries             ; (0000:7C11=70h) ax = data * ax
        mov     bx,200h                 ; 512 bytes per sector
        add     ax,bx
        dec     ax
        div     bx                      ; ax,dx rem=dx:ax/reg
        add     sec_to_get,ax           ; (0000:7C2E) now ptr to start of data area

; ----- get set to read directory into buffer (BIOS communications area)
;
        mov     ax,50h                  ; set ES to BIOS communications
                                        ;  area at 0050:0000
        mov     es,ax
        mov     ax,directory_sec        ; (0000:7C2C) get directory logical sector
        call    calc_C_H_N              ; (7D44)
        xor     bx,bx                   ; ES:BX is 0050:0000 BIOS communicatio
                                        ; ns area
        mov     ax,201h                 ; int 13h: read one sector
        call    read_secs               ; (7D5E)
        jc      Nonsys_reboot           ; Jump if carry Set

; ----- compare first 11-byte directory entry against 'IBMBIO  COM'
;
        xor     di,di                   ; points to directory entries in
                                        ;  BIOS comm area
        mov     cx,0Bh                  ; compare first 11-byte directory
                                        ;  entry looking for 'IBMBIO  COM'
        mov     si,offset ibmbio_str    ; (0000:7DD6=49h)
        cld                             ; Clear direction
        repe    cmpsb                   ; Rep zf=1+cx >0 Cmp [si] to es:[di]
        jnz     Nonsys_reboot           ; Jump if not zero

; ----- compare second directory entry against 'IBMDOS  COM'
;
        mov     di,offset dir_2nd_entry        ; point to next dir entry ('IBMDOS
                                        ;   COM')
        mov     si,offset ibmdos_str    ; (0000:7DE1=49h)
        mov     cx,0Bh                  ; compare next 11-byte directory
                                        ;  entry looking for 'IBMDOS  COM'
        repe    cmpsb                   ; Rep zf=1+cx >0 Cmp [si] to es:[di]
        jz      setup_ibmbio_ld         ; Jump if zero

; ----- emit error message and reboot if failed disk read
;
Nonsys_reboot:
        mov     si,offset Nonsys_msg    ; (0000:7D77=0Dh)
        call    tty_msg                 ; (7D36)
        xor     ah,ah                   ; Zero register
        int     16h                     ; Keyboard i/o  ah=function 00h
                                        ;  get keybd char in al, ah=scan
        pop     si
        pop     ds
        pop     word ptr [si]
        pop     word ptr [si+2]
        int     19h                     ; Bootstrap loader

; ----- First two directory names ok, so proceed to load ibmbio.com
;
setup_ibmbio_ld:
        mov     ax,es:ibmbio_len        ; (0050:001C)
        mov     bx,200h                 ; 512 bytes per sector
        xor     dx,dx                   ; Zero register
        div     bx                      ; ax,dx rem=dx:ax/reg
        inc     al
        mov     secs_ibmbio,al          ; (0000:7C20=14h)
        mov     ax,sec_to_get           ; (0000:7C2E) now ptr to start of data area
        mov     ibmbio_sec,ax           ; points to ibmbio.com logical sector
        mov     ax,70h                  ; load point for ibmbio.com (0000:0700
                                        ;  or 0070:0000)
        mov     es,ax
        xor     bx,bx                   ; Zero register

get_next_sector:
        mov     ax,sec_to_get           ; (0000:7C2E) next logical sector #
        call    calc_C_H_N              ; (7D44)
        mov     al,byte ptr secs_per_trk        ; (0000:7C18=9)
        sub     al,N_sector                     ; (0000:7C16=2) N_sector now
        inc     al
        xor     ah,ah                   ; Zero register
        push    ax
        mov     ah,2
        call    read_secs               ; (7D5E)
        pop     ax
        jc      DBfail_freeze           ; Jump if carry Set
        sub     secs_ibmbio,al          ; (0000:7C20=14h)
        jbe     chain_to_ibmbio         ; Jump if below or =
        add     sec_to_get,ax           ; (0000:7C2E) update logical sector pointer
        mul     bytes_per_sec           ; (0000:7C0B=200h) ax = data * ax
        add     bx,ax                   ; bx points to next sector load offset
        jmp     short get_next_sector   ; (7CF1)
chain_to_ibmbio:
        mov     ch,format_id            ; (0000:7C15=0FDh)
        mov     dl,drive                ; (0000:7C1E=0)
        mov     bx,ibmbio_sec           ; points to ibmbio.com logical sector
;*              jmp     far ptr ibmbio_entry    ;*(0070:0000)
        db      0EAh, 00h, 00h, 70h, 00h
DBfail_freeze:
        mov     si,offset DBfail_msg    ; (0000:7DC0=0Dh)
        call    tty_msg                 ; (7D36)
freeze:
        jmp     short freeze            ; this freezes things up

booti300          endp

;
;         [si]: ASCIIZ string to console
;

tty_msg         proc    near
loc_5:
        lodsb                           ; String [si] to al
        test    al,0FFh                 ; skip if null string
        jz      loc_ret_6               ; Jump if zero
        mov     ah,0Eh
        mov     bx,7
        int     10h                     ; Video display   ah=functn 0Eh
                                        ;  write char al, teletype mode
        jmp     short loc_5             ; (7D36)

; External Entry into Subroutine 
;
;         cvt logical sector in ax to C-H-N

calc_C_H_N:
        xor     dx,dx                   ; Zero register
        div     secs_per_trk            ; (0000:7C18=9) ax,dxrem=dx:ax/data
        inc     dl
        mov     N_sector,dl             ; N_sector -- save it
        xor     dx,dx                   ; Zero register
        div     sides                   ; (0000:7C1A=2) ax,dxrem=dx:ax/data
        mov     H_head,dl               ; (0000:7C1F=0) save it
        mov     C_track,ax              ; (0000:7C30) save it

loc_ret_6:
        retn
tty_msg         endp


;
;         BIOS int 13h diskette read C-H-N
;

read_secs       proc    near
        mov     dx,C_track              ; (0000:7C30)
        mov     cl,6
        shl     dh,cl                   ; Shift w/zeros fill
        or      dh,N_sector             ; this is N_Sector
        mov     cx,dx
        xchg    ch,cl
        mov     dx,word ptr drive       ; (0000:7C1E=0)
        int     13h                     ; Disk  dl=drive a  ah=func 02h
                                        ;  read sectors to memory es:bx
        retn
read_secs       endp

ibmbio_sec      dw      0
Nonsys_msg      db      0Dh, 0Ah, 'Non-System disk or dis'
                db      'k error', 0Dh, 0Ah, 'Replace and'
                db      ' strike any key when ready', 0Dh
                db      0Ah, 0
DBfail_msg      db      0Dh, 0Ah, 'Disk Boot failure', 0Dh
                db      0Ah, 0
ibmbio_str      db      'IBMBIO  COM'
ibmdos_str      db      'IBMDOS  COM'
                db      18 dup (0)
                db       55h,0AAh

seg_a           ends



                end     start

;  finis 

