
PAGE  80,132

; BOOTI330.ASM  Boot sector - IBM PC-DOS 3.30
;
; 7.9.91        Disassembly from floppy boot sector 512-byte image (lsz)
;               Assembles error-free (MASM 5.0)
;               Byte-identical to IBM PC DOS 3.30 floppy boot sector
;               Not tested

; ------------------- program notes -------------------------------------
comment ~

This is a disassembly of the boot sector of an IBM PC-DOS 3.30 diskette.

Program control is transferred to this code by the system BIOS
after it completes its POST routines.

This code loads io.sys and then chains to it.

To create a boot sector image from this .asm file, first go
through the usual exercise of creating an .exe file, where
ww.asm is this working source file:

       C:\>masm ww,,ww;
       C:\>link ww,ww;

The boot sector code is loaded by the system BIOS at 0000:7c00.
This .asm file will generate a load image that has the proper
offsets and references at 7c00h.  However, since MASM does not
generate by itself binary images and exe2bin does not convert
to binary  modules .exe files that have been org'ed to anything
other than 100h, the 512-byte binary image has to be manually
extracted from the full segment image produced by the linker
(7c00h+200h=7e00h or 32256 bytes).

To generate only the 512-byte sector image, run a small .bat
file with the following code:

     echo nww.bin      >cxe2bin.arf
     echo rcx          >>cxe2bin.arf
     echo 200          >>cxe2bin.arf
     echo w7c00        >>cxe2bin.arf
     echo q            >>cxe2bin.arf
     debug ww.exe      <cxe2bin.arf

When the ww.bin file created above is compared with an image
of the diskette boot sector, it will be byte-for-byte identical
except for the machine- and disk-format specific data bytes.

 R. Zider
 San Francisco

~tnemmoc


; ----- Data segment ---------------------------------------------------------
;
;       This section is to establish addressability of locations.
;       No code gets generated by the assembler.
;
dseg            segment at 0000
                org     1eh*4                   ; int 1Eh is ptr to disk parms
int_1Eh         dd      ?                       ; Int 1Eh: ptr to diskette parms
                org     500h
bioscommarea    db      ?                       ; BIOS communications area
                org     51Ch
ibmbio_len      dw      ?                       ; file length within dir entry
                org     700h
ibmbio_loadpt   db      ?                       ; (0000:0700)


; ----- labels for working storage locs that overlay initial code area

                org     7c37h
sec_to_get      dw      ?                       ; logical sector number
C_track         dw      ?                       ; cylinder in C-H-N
N_sector        db      ?                       ; N in C-H-N
secs_ibmbio     db      ?                       ; length of ibmbio in sectors
ibmbio_sec      dw      ?                       ; logical sec # of ibmbio.com
directory_sec   dw      ?                       ; logical sec # of directory

dseg            ends


; ------------- Code segment ---------------------------------------------
;
seg_a           segment byte public
                assume  cs:seg_a, ds:seg_a


                org     7c00h                   ; code at 0000:7c00

boot330         proc    far

start:
                jmp     short loc_1             ; (7C36)
                db      90h

; ------------- data section
;
oem_ver         db      'IBM  3.3'
bytes_per_sec   dw      200h                   ; 512 bytes/sector
secs_per_clust  db      2
reserv_secs     dw      1
fat_copies      db      2
dir_entries     dw      70h                    ; 112 dir entries
secs_on_disk    dw      2D0h                   ; 720 sectors
format_id       db      0FDh                   ; DSDD 5.25" 360KB diskette
secs_per_FAT    dw      2
secs_per_trk    dw      9
sides           dw      2
hidden_secs     dw      0
                db      12 dup (0)
H_head          db      0

; ----- diskette parameter table
;
disk_parms      db      00              ; 1st specify byte
                db      00              ; 2nd specify byte
                db      00              ; motor wait time
                db      00              ; 512 bytes per sector
                db      12h             ; EOT (last sector on track)
                db      00              ; gap length
                db      00              ; DTL
                db      00              ; gap length for format
                db      00              ; fill byte for format
                db      01              ; head settle time (ms) <== machine dependent **
                db      00              ; motor start time (1/8 secs)

; ------------- code section
;
loc_1:                                  ; program starts here

; ----- set up local stack from load point down
;
        cli                             ; Disable interrupts while stack setup
        xor     ax,ax                   ; Zero register
        mov     ss,ax                   ; SS to 0000
        mov     sp,offset start
        push    ss
        pop     es                      ; ES to 0000

; ----- load disk parameter table and set new int 1Eh
;
        mov     bx,offset int_1Eh     ; (0000:0078) BIOS disk parms ptr
        lds     si,dword ptr ss:[bx]    ; Load 32 bit ptr
        push    ds                      ; save old ptr on stack
        push    si
        push    ss                      ; save ptr to int 1Eh as well
        push    bx
        mov     di,offset disk_parms    ; table to be loaded
        mov     cx,0Bh                  ; copy 11-byte disk parm table (BPB)
        cld                             ; Clear direction

locloop_2:
        lodsb                           ; String [si] to al
        cmp     byte ptr es:[di],0      ; don't overlay non-zero values
        je      loc_3                   ; Jump if equal
        mov     al,es:[di]              ; use old value if non-zero
loc_3:
        stosb                           ; Store al to es:[di]
        mov     al,ah                   ; zero out al
        loop    locloop_2               ; Loop if cx > 0

        push    es                      ; DS to 0000
        pop     ds
        mov     [bx+2],ax               ; new int 1Eh seg = 0000
        mov     word ptr [bx],offset disk_parms ; new int 1Eh offset
        sti                             ; Enable interrupts

; ----- reset boot diskette drive a:
;
        int     13h                     ; Disk  dl=drive a  ah=func 00h
                                        ;  reset disk, al=return status
        jc      DBfail_reboot           ; Jump if carry Set

; ----- calc starting logical sector for directory
;
        mov     al,fat_copies           ; (0000:7C10=2)
        cbw                             ; Convrt byte to word
        mul     secs_per_FAT            ; (0000:7C16=2) ax = data * ax
        add     ax,hidden_secs          ; (0000:7C1C=0)
        add     ax,reserv_secs          ; (0000:7C0E=1)
        mov     directory_sec,ax        ; ptr to start of directory
        mov     sec_to_get,ax           ; cumulate

; ----- now calc starting logical sector of data area
;
        mov     ax,20h
        mul     dir_entries             ; (0000:7C11=70h) ax = data * ax
        mov     bx,bytes_per_sec        ; (0000:7C0B=200h)
        add     ax,bx
        dec     ax
        div     bx                      ; ax,dx rem=dx:ax/reg
        add     sec_to_get,ax           ; now ptr to start of data area

; ----- get set to read directory into buffer (BIOS communications area)
;
        mov     bx,offset bioscommarea  ; (0000:0500) use to hold directory entries
        mov     ax,directory_sec        ; get directory
        call    calc_C_H_N              ; (7D40)
        mov     ax,201h                 ; int 13h: read one sector
        call    read_secs               ; (7D5A)
        jc      Nonsys_reboot           ; Jump if carry Set

; ----- compare first 11-byte directory entry against 'IBMBIO  COM'
;
        mov     di,bx                   ; points to directory entries
        mov     cx,0Bh                  ; compare 11 bytes
        mov     si,offset ibmbio_str    ; (0000:7DBE='IBMBIO  COM')
        repe    cmpsb                   ; Rep zf=1+cx >0 Cmp [si] to es:[di]
        jnz     Nonsys_reboot           ; Jump if not zero

; ----- compare second directory entry against 'IBMDOS  COM'
;
        lea     di,[bx+20h]             ; Load effective addr--next dir entry
        mov     si,offset ibmdos_str    ; (0000:7DE1=49h)
        mov     cx,0Bh                  ; compare 11 bytes
        repe    cmpsb                   ; Rep zf=1+cx >0 Cmp [si] to es:[di]
        jz      setup_ibmbio_ld         ; Jump if compares ok

; ----- emit error message and reboot if failed disk read
;
Nonsys_reboot:                                  ; Non-system diskette case
        mov     si,offset Nonsys_msg    ; (0000:7D77=0Dh)
loc_4:
        call    tty_msg                 ; (7D32)
        xor     ah,ah                   ; Zero register
        int     16h                     ; Keyboard i/o  ah=function 00h
                                        ;  get keybd char in al, ah=scan
        pop     si
        pop     ds
        pop     word ptr [si]           ; restore old int 1Eh ptr to disk parms
        pop     word ptr [si+2]
        int     19h                     ; Bootstrap loader

DBfail_reboot:                                  ; Disk Boot failed case
        mov     si,offset DBfail_msg    ; (0000:7DC0=0Dh)
        jmp     short loc_4             ; (7CC5)

; ----- First two directory names ok, so proceed to load ibmbio.com
;
setup_ibmbio_ld:
        mov     ax,ds:ibmbio_len        ; (0000:051C) ibmbio length in bytes
        xor     dx,dx                   ; Zero register
        div     bytes_per_sec           ; (0000:7C0B=200h) ax,dxrem=dx:ax/data
        inc     al                      ; bump count to round up
        mov     secs_ibmbio,al          ; (0000:7C3C) save length of ibmbio in secs
        mov     ax,sec_to_get           ; (0000:7C37) now ptr to start of data area
        mov     ibmbio_sec,ax           ; (0000:7C3D) save it
        mov     bx,offset ibmbio_loadpt        ; load point for ibmbio.com (0000:0700
                                        ;  or 0070:0000)
get_next_sector:
        mov     ax,sec_to_get           ; (0000:7C37h) get next logical sector #
        call    calc_C_H_N              ; (7D40) convert to physical C-H-N
        mov     ax,secs_per_trk         ; (0000:7C18=9)
        sub     al,N_sector             ; (0000:7C3B) sector in C-H-N
        inc     ax
        cmp     secs_ibmbio,al          ; ** new limit check here **
                                        ; don't get any more than exact file size
        jae     loc_5                   ; Jump if above or =
        mov     al,secs_ibmbio          ; (0000:7C3C)
loc_5:
        push    ax
        call    read_secs               ; (7D5A)
        pop     ax
        jc      DBfail_reboot           ; Jump if carry Set
        sub     secs_ibmbio,al          ; save remaining len to read      (0000:7C3C)
        jz      chain_to_ibmbio         ; ** jz now, not jbe ** will be exact
        add     sec_to_get,ax           ; update logical sector pointer   (0000:7C37)
        mul     bytes_per_sec           ; (0000:7C0B=200h) ax = data * ax
        add     bx,ax
        jmp     short get_next_sector   ; (7CF1)

; ----- ibmbio.com now loaded, so transfer control to it at 0070:0000
;
chain_to_ibmbio:
        mov     ch,format_id            ; (0000:7C15=0FDh)
        mov     dl,drive                ; (0000:7DFD=0)
        mov     bx,ibmbio_sec           ; points to ibmbio.com logical sector
;*              jmp     far ptr ibmbio_entry    ;*(0070:0000)
        db      0EAh, 00h, 00h, 70h, 00h

boot330          endp

;
;                              SUBROUTINE
;

tty_msg         proc    near
loc_6:
        lodsb                           ; String [si] to al
        or      al,al                   ; Zero ?
        jz      loc_ret_7               ; Jump if zero
        mov     ah,0Eh                  ; video tty function
        mov     bx,7
        int     10h                     ; Video display   ah=functn 0Eh
                                        ;  write char al, teletype mode
        jmp     short loc_6             ; (7D32)

; External Entry into Subroutine 
;
; Logical sector to physical C-H-N conversion

calc_C_H_N:
        xor     dx,dx                   ; Zero register
        div     secs_per_trk            ; (0000:7C18=9) ax,dxrem=dx:ax/data
        inc     dl
        mov     N_sector,dl             ; N_sector -- save it
        xor     dx,dx                   ; Zero register
        div     sides                   ; (0000:7C1A=2) ax,dxrem=dx:ax/data
        mov     H_head,dl               ; (0000:7C2A=0)
        mov     C_track,ax              ; C_track -- saveit

loc_ret_7:
        retn
tty_msg         endp


;
;                              SUBROUTINE
;

read_secs       proc    near
        mov     ah,2                    ; ah=02: int 13h read sector function
        mov     dx,C_track              ; (0000:7C39)
        mov     cl,6
        shl     dh,cl                   ; Shift w/zeros fill
        or      dh,N_sector             ; (0000:7C3B)
        mov     cx,dx
        xchg    ch,cl
        mov     dl,drive                ; (0000:7DFD=0)
        mov     dh,H_head               ; (0000:7C2A=0)
        int     13h                     ; Disk  dl=drive a  ah=func 02h
                                        ;  read sectors to memory es:bx
        retn
read_secs       endp

Nonsys_msg      db      0Dh, 0Ah, 'Non-System disk or dis'
                db      'k error', 0Dh, 0Ah, 'Replace and'
                db      ' strike any key when ready', 0Dh
                db      0Ah, 0
DBfail_msg      db      0Dh, 0Ah, 'Disk Boot failure', 0Dh
                db      0Ah, 0
ibmbio_str      db      'IBMBIO  COM'
ibmdos_str      db      'IBMDOS  COM'
                db      17 dup (0)
drive           db      0
                db       55h,0AAh

seg_a           ends


                end     start

;  finis 
