
PAGE  80,132

; BOOTM500.ASM  Boot sector - MS DOS 5.00
;
; 7.9.91        Disassembly from floppy boot sector 512-byte image (lsz)
;               Assembles error-free (MASM 5.0)
;               Byte-identical to MS DOS 5.0 floppy boot sector
;               Not tested

; ------------------- program notes -------------------------------------
comment ~

This is a disassembly of the boot sector of an MS DOS 5.00 diskette.

Program control is transferred to this code by the system BIOS
after it completes its POST routines.

This code loads io.sys and then chains to it.

To create a boot sector image from this .asm file, first go
through the usual exercise of creating an .exe file, where
ww.asm is this working source file:

       C:\>masm ww,,ww;
       C:\>link ww,ww;

The boot sector code is loaded by the system BIOS at 0000:7c00.
This .asm file will generate a load image that has the proper
offsets and references at 7c00h.  However, since MASM does not
generate by itself binary images and exe2bin does not convert
to binary  modules .exe files that have been org'ed to anything
other than 100h, the 512-byte binary image has to be manually
extracted from the full segment image produced by the linker
(7c00h+200h=7e00h or 32256 bytes).

To generate only the 512-byte sector image, run a small .bat
file with the following code:

     echo nww.bin      >cxe2bin.arf
     echo rcx          >>cxe2bin.arf
     echo 200          >>cxe2bin.arf
     echo w7c00        >>cxe2bin.arf
     echo q            >>cxe2bin.arf
     debug ww.exe      <cxe2bin.arf

When the ww.bin file created above is compared with an image
of the diskette boot sector, it will be byte-for-byte identical
except for the machine- and disk-format specific data bytes.

 R. Zider
 San Francisco

~tnemmoc

; ----- Data segment ---------------------------------------------------------
;
;       This section is to establish addressability of locations.
;       No code gets generated by the assembler.
;
dseg            segment at 0000
                org     1eh*4                   ;
int_1Eh         dd      ?                       ; Int 1Eh: ptr to diskette parms
                org     500h
bioscommarea    db      ?                       ; BIOS communications area
                org     51Ch
ibmbio_len      dw      ?                       ; file length within dir entry
                org     700h
ibmbio_loadpt   db      ?                       ; ibmbio.com/io.sys load point


; ----- labels for working storage locs that overlay initial code area
;
; ----- diskette parameter table
;       The values in the below parm table are typical for
;       a 360KB DSDD diskette drive in an AT
                org     7c3eh
disk_parms      db      0dfh             ; 1st specify byte
                db      02h             ; 2nd specify byte
                db      25h             ; motor wait time
                db      02h             ; 512 bytes per sector
                db      0fh             ; EOT (last sector on track)
                db      1bh             ; gap length
                db      0ffh             ; DTL
                db      54h             ; gap length for format
                db      0f6h             ; fill byte for format
                db      01              ; head settle time (ms) <== machine dependent **
                db      08h             ; motor start time (1/8 secs)

; ------------- working temporary storage
                org     7c49h
sec_to_get_lsw  dw      ?                       ; logical sector number low word
sec_to_get_msw  dw      ?                       ; logical sector number high word
C_track         dw      ?                       ; cylinder in C-H-N
N_sector        db      ?                       ; N in C-H-N
dir_sec_lsw     dw      ?                       ; logical sec # of directory - low word
dir_sec_msw     dw      ?                       ; logical sec # of directory - high word

dseg            ends


; ------------- Code segment ---------------------------------------------
;
seg_a           segment byte public
        assume  cs:seg_a, ds:seg_a


                org     7c00h                   ; code at 0000:7c00

boot500         proc    far

start:
                jmp     short loc_1             ; (7C3E)
                db      90h

; ------------- data section
;
oem_ver         db      'MSDOS5.0'
bytes_per_sec   dw      200h                   ; 512 bytes/sector
secs_per_clust  db      1
reserv_secs     dw      1
fat_copies      db      2
dir_entries     dw      0E0h                   ; 224 dir entries
secs_on_disk    dw      960h                   ; 2400 sectors
format_id       db      0F9h                   ; DSHD 5.25" AT diskette
secs_per_FAT    dw      7                      ; 7 sectors per FAT copy
secs_per_trk    dw      0Fh                    ; 15 sectors/track
sides           dw      2
hidden_secs_lsw dw      0
hidden_secs_msw dw      0
secs_save       dw      0
                db      0, 0
drive           db      0
H_head          db      0
                db       29h,0CEh, 17h, 6Ch, 07h
                db      'DOS 500    FAT12   '

; -------------- code section
;
loc_1:
; ----- set up local stack from load point down
        cli                             ; Disable interrupts
        xor     ax,ax                   ; Zero register
        mov     ss,ax
        mov     sp,offset start
        push    ss
        pop     es

; ----- load disk parameter table and set new int 1Eh
        mov     bx,offset int_1Eh       ; (0000:0078) BIOS disk parms ptr
        lds     si,dword ptr ss:[bx]    ; Load 32 bit ptr
        push    ds                      ; save old ptr on stack
        push    si
        push    ss                      ; save ptr to int 1Eh as well
        push    bx
        mov     di,offset disk_parms    ; table to be loaded (0000:7C3E)
        mov     cx,0Bh                  ; copy 11-byte disk parm table
        cld                             ; Clear direction
        rep     movsb                   ; Rep when cx >0 Mov [si] to es:[di]
        push    es
        pop     ds

; ----- overlay parm table with specific values
        mov     byte ptr [di-2],0Fh     ; head settle time
        mov     cx,secs_per_trk         ; (0000:7C18=0Fh)
        mov     [di-7],cl               ; EOT (max sector number)
        mov     [bx+2],ax               ; new int 1Eh seg = 0000
        mov     word ptr [bx],offset disk_parms ; save ptr to new disk parms table
        sti                             ; Enable interrupts

; ----- reset boot diskette drive a:
        int     13h                     ; Disk  dl=drive a  ah=func 00h
                                        ;  reset disk, al=return status
        jc      Nonsys_reboot           ; Jump if carry Set

; ----- calc starting logical sector for directory
        xor     ax,ax                   ; Zero register
        cmp     secs_on_disk,ax         ; (0000:7C13=960h)
        je      loc_4
        mov     cx,secs_on_disk         ; (0000:7C13=960h)
        mov     secs_save,cx            ; (0000:7C20=0)
loc_4:
        mov     al,fat_copies           ; (0000:7C10=2)
        mul     secs_per_FAT            ; (0000:7C16=7) ax = data * ax
        add     ax,hidden_secs_lsw      ; (0000:7C1C=0)
        adc     dx,hidden_secs_msw      ; (0000:7C1E=0)
        add     ax,reserv_secs          ; (0000:7C0E=1)
        adc     dx,0
        mov     dir_sec_lsw,ax          ; (0000:7C50)
        mov     dir_sec_msw,dx          ; (0000:7C52)
        mov     sec_to_get_lsw,ax       ; (0000:7C49)
        mov     sec_to_get_msw,dx       ; (0000:7C4B)
        mov     ax,20h
        mul     dir_entries             ; (0000:7C11=0E0h) ax = data * ax
        mov     bx,bytes_per_sec        ; (0000:7C0B=200h)
        add     ax,bx
        dec     ax
        div     bx                      ; ax,dx rem=dx:ax/reg
        add     sec_to_get_lsw,ax       ; (0000:7C49)
        adc     sec_to_get_msw,0        ; (0000:7C4B)

; ----- get set to read directory into buffer (BIOS communications area)
        mov     bx,offset bioscommarea         ; (0000:0500=0)
        mov     dx,dir_sec_msw          ; (0000:7C52)
        mov     ax,dir_sec_lsw          ; (0000:7C50)
        call    calc_C_H_N              ; (7D60)
        jc      Nonsys_reboot           ; Jump if carry Set
        mov     al,1                    ; read one sector
        call    read_secs               ; (7D81)
        jc      Nonsys_reboot           ; Jump if carry Set

; ----- compare first 11-byte directory entry against 'IO      SYS'
        mov     di,bx
        mov     cx,0Bh                  ; compare 11 bytes
        mov     si,offset ibmbio_str    ; (0000:7DE6=49h)
        repe    cmpsb                   ; Rep zf=1+cx >0 Cmp [si] to es:[di]
        jnz     Nonsys_reboot           ; Jump if not zero

; ----- compare second directory entry against 'MSDOS   SYS'
        lea     di,[bx+20h]             ; Load effective addr
        mov     cx,0Bh                  ; compare 11 bytes
        repe    cmpsb                   ; Rep zf=1+cx >0 Cmp [si] to es:[di]
        jz      setup_ibmbio_ld         ; Jump if zero

; ----- emit error message and reboot if failed disk read
;
Nonsys_reboot:
        mov     si,offset Nonsys_msg    ; (0000:7D9E=0Dh)
        call    tty_msg                 ; (7D52)
        xor     ax,ax                   ; Zero register
        int     16h                     ; Keyboard i/o  ah=function 00h
                                        ;  get keybd char in al, ah=scan
        pop     si                      ; restore registers from stack
        pop     ds
        pop     word ptr [si]           ; restore old int 1Eh ptr to disk parms
        pop     word ptr [si+2]
        int     19h                     ; Bootstrap loader
xxFail_reboot:
        pop     ax
        pop     ax
        pop     ax
        jmp     short Nonsys_reboot     ; (7CED)

; ----- First two directory names ok, so proceed to load ibmbio.com (io.sys)
;       Note that ibmio.com/io.sys now is not required to be first file in
;       the data area, since the below code now picks up pointer to
;       the file from the directory entry at offset 1Ah.
setup_ibmbio_ld:
        mov     ax,[bx+1Ah]             ; starting cluster number of io.sys
        dec     ax                      ; subtract 2 to get offset from
        dec     ax                      ;   start of data area
        mov     bl,secs_per_clust       ; (0000:7C0D=1)
        xor     bh,bh                   ; Zero register
        mul     bx                      ; dx:ax = reg * ax
        add     ax,sec_to_get_lsw       ; (0000:7C49)
        adc     dx,sec_to_get_msw       ; (0000:7C4B=0C536h)
        mov     bx,offset ibmbio_loadpt        ; (0000:0700=0BDh)
        mov     cx,3                    ; load first three sectors only
get_next_sector:
        push    ax
        push    dx
        push    cx
        call    calc_C_H_N              ; (7D60)
        jc      xxFail_reboot           ; Jump if carry Set
        mov     al,1
        call    read_secs               ; (7D81)
        pop     cx
        pop     dx
        pop     ax
        jc      Nonsys_reboot           ; Jump if carry Set
        add     ax,1                    ; increment for next sector
        adc     dx,0
        add     bx,bytes_per_sec        ; bx points to next sector load
                                        ;  offset
        loop    get_next_sector         ; Loop if cx > 0

; ----- first three sectors of ibmbio.com (io.sys) now loaded, so transfer
;       control to it at 0070:0000
        mov     ch,format_id            ; (0000:7C15=0F9h)
        mov     dl,drive                ; (0000:7C24=0)
        mov     bx,sec_to_get_lsw       ; points to ibmbio.com (io.sys)
                                        ;  logical sector (0000:7C49)
        mov     ax,sec_to_get_msw       ; (0000:7C4B)
;*      jmp     far ptr ibmbio_entry    ;*(0070:0000)
        db      0EAh, 00h, 00h, 70h, 00h

boot500         endp

;
;                              SUBROUTINE
;

tty_msg         proc    near
loc_5:
        lodsb                           ; String [si] to al
        or      al,al                   ; Zero ?
        jz      loc_ret_7               ; Jump if zero
        mov     ah,0Eh                  ; video tty function
        mov     bx,7
        int     10h                     ; Video display   ah=functn 0Eh
                                        ;  write char al, teletype mode
        jmp     short loc_5             ; (7D52)

; External Entry into Subroutine 
;
; Logical sector to physical C-H-N conversion

calc_C_H_N:
        cmp     dx,secs_per_trk         ; (0000:7C18=0Fh) check against
                                        ; integer divide overflow
        jae     loc_6                   ; Jump if above or =
        div     secs_per_trk            ; (0000:7C18=0Fh) ax,dxrem=dx:ax/data
        inc     dl
        mov     N_sector,dl             ; N_sector -- save it
        xor     dx,dx                   ; Zero register
        div     sides                   ; (0000:7C1A=2) ax,dxrem=dx:ax/data
        mov     H_head,dl               ; (0000:7C25=0)
        mov     C_track,ax              ; C_track -- saveit
        clc                             ; Clear carry flag
        retn
loc_6:
        stc                             ; Set carry flag

loc_ret_7:
        retn
tty_msg         endp


;
;                              SUBROUTINE
;

read_secs       proc    near
        mov     ah,2                    ; ah=02: int 13h read sector function
        mov     dx,C_track              ; (0000:7C4D=1E37h)
        mov     cl,6
        shl     dh,cl                   ; Shift w/zeros fill
        or      dh,N_sector             ; (0000:7C4F=56h)
        mov     cx,dx
        xchg    ch,cl
        mov     dl,drive                ; (0000:7C24=0)
        mov     dh,H_head               ; (0000:7C25=0)
        int     13h                     ; Disk  dl=drive a  ah=func 02h
                                        ;  read sectors to memory es:bx
        retn
read_secs       endp

Nonsys_msg      db      0Dh, 0Ah, 'Non-System disk or dis'
                db      'k error', 0Dh, 0Ah, 'Replace and'
                db      ' press any key when ready', 0Dh, 0Ah
                db      0
; ----- these two entries now must be one after the other
ibmbio_str      db      'IO      SYS'
ibmdos_str      db      'MSDOS   SYS'
                db       00h, 00h, 55h,0AAh

seg_a           ends



                end     start


;  finis 

