page 88,132

; BOOTR20.ASM   Boot record - DOS 2.0
;
; 7.5.91        Reformatted and cleaned up (lsz)
;               Assembles error-free
;               Byte-identical to DOS 2.0 floppy boot sector with one byte noted
;               Not tested
; 7.4.91        Downloaded as bootr20.zip within bootz.zip
;                 from VOR.  (Boot package from Vern Buerg)
; 3.14.86	Original .asm file date.  No author noted.


; ------------------- program notes -------------------------------------
;
; This is a disassembly of the boot sector of a DOS 2.0 diskette.
;
; Program control is transferred to this code by the system BIOS
; after it completes its POST routines.
; 
; This code loads ibmbio.com and then chains to it.
; 
; To create a boot sector image from this .asm file, go through
; the usual exercise of creating a .com file, where ww.asm is the
; working source file (this file -- bootr20.asm).
;
;	C:\>masm ww,,ww;
;	C:\>link ww,ww;
;	C:\>exe2bin ww
;	C:\>del ww.exe
;
; When the ww.bin file created above is compared with DOS 2.0, the only
; discrepancy from a DOS 2.0 boot sector is the head settle time byte 
; at file offset 02Ah.  The code below specifies 0fh; my machines
; generate a zero at this byte.  Other than that, the reassembly is
; byte-identical with the boot sector.
;
; R. Zider
; San Francisco
; ===========================================================================


cseg            SEGMENT PARA    PUBLIC  'CODE'
                ASSUME  CS:cseg,DS:cseg,SS:cseg,ES:cseg

; ----- Use 100h so we can run this through exe2bin (no stack segment)
                ORG     100h

; The boot record is loaded in at 07c0:0000 (0000:7C00) by POST and executed.
; The data offsets are all in relation to DS = 0 so all offsets must
; be increased by 7b00 (7c00 - 100).
;
load_point              EQU     7b00h

; These equates allow a convenient way to refer to data in the boot sector
;
location_of             EQU     load_point+offset
data_at                 EQU     load_point+


; ----------------- beginning of program --------------------------
;
; The POST system has finished and the boot sector
; is read into 0000:7c00.  The registers are still set from
; that read.  Control is transferred to code at 0000:7c00h (enter).
;
enter:                  JMP     main            ;skip over data area

; ------------------ data area ------------------------------------
;
track_word              equ     this word
track                   equ     this word+5
version                 DB      'IBM  2.0'
bytes_per_sector        DW      200h            ; 512-byte sectors
sects_per_clust         DB      2
reserv_sects            DW      1
fat_copies              DB      2
dir_entries             DW      70h             ;70h = 112 dir entries
cum_sects               equ     this word
sects_on_disk           DW      02d0h           ;02d0h = 720 sectors
format_id               equ     this byte
sector                  DB      0fdh
sect_per_fat            DW      2
sect_per_trk            equ     this byte
sec_per_trk_word        DW      9
sides                   DW      2
hidden_sectors          DW      0

head_drive              equ     this word
drive                   DB      0
head                    DB      0
max_secs_ibmbio         DB      0ah             ; 4608/512=9, but 2 sec/cl=>10
                                                ; (crippled masm 5.0 arithmetic
                                                ; does not allow forming a
                                                ; general expression here)
disk_params             DB      0dfh            ; 1st specify byte
                        DB      2               ; 2nd specify byte
                        DB      25h             ; motor wait time
                        DB      2               ; 512 bytes per sector
                        DB      9               ; EOT (last sector on track)
                        DB      02ah            ; gap length
                        DB      0ffh            ; DTL
                        DB      050h            ; gap length for format
                        DB      0f6h            ; fill byte for format
                        DB      0fh             ; head settle time (ms) <== machine dependent **
                        DB      02              ; motor start time (1/8 secs)


; --------------------------------------------------------------
re_boot:
;
        INT     19h         ; start boot over again


; --------------------------------------------------------------
main    PROC    NEAR
;

; ----- set up local stack area just below load point
;       protect SP change from interrupts
        CLI                 ; hold interrupts
        XOR     AX,AX       ; zero ax
        MOV     SS,AX       ; set stack at segment 0
        MOV     SP,location_of enter  ; set stack pointer to entry (7C00h)
        MOV     DS,AX       ; set data segment to 0

; ----- set int 1Eh disk base ptr to point to disk parameter table
;       at 0000:7c21 (1Eh*4 = 0000:0078h)
        MOV     DS:[007ah],AX                           ; segment=0000
        MOV     DS:[0078h],location_of disk_params      ; offset=7C21h
        STI                                             ; ok to interrupt

; ----- do an int 13h disk reset (ah = 0), floppy drive a: (dl = 0)
        INT     13h
        JNB     disk_is_there                           ; reset ok
        JMP     boot_failure

disk_is_there:
; ----- get the number of sectors preceding first directory sector
;       = hidden + reserved (boot) + FATs
        PUSH    CS                                  ; reset cseg addressability
        POP     DS                                  ; now DS=CS
        MOV     AL,data_at fat_copies               ; value 2
        CBW
        MUL     WORD PTR data_at sect_per_fat       ; value 2
        ADD     AX,data_at hidden_sectors           ; value 0
        ADD     AX,data_at reserv_sects             ; value 1
        MOV     data_at track_word,AX               ; moves a 5 here (tmp)
        MOV     data_at cum_sects,AX                ; cum sectors so far
        MOV     AX,0020h                            ; 32 bytes per dir entry
        MUL     DS:[data_at dir_entries]            ; 70h (70h*20h)=0e00h
        ADD     AX,01ffh                            ; plus 511 gets you fffh
        MOV     BX,0200h                            ; 512 bytes/sector
        DIV     BX                                  ; leaves you 7 secs/dir
        ADD     DS:[data_at cum_sects],AX           ; makes it 12 cum

; ----- check for ibmbio, ibmdos as first two directory entries
        CALL    check_sys
        JB      re_boot                             ; if CY then try again

; ----- setup for read of ibmbio.com located at first sector of data area
        MOV     AX,DS:[data_at cum_sects]           ; get logical sector of ibmbio
        MOV     DS:[data_at sector_to_read],AX      ; save it
        MOV     AX,0070h                            ; where ibmbio gets loaded
        MOV     ES,AX
        MOV     DS,AX
        MOV     BX,0000h                            ; now read will go to 70:0000

; ----- read ibmbio.com from first sector of data area (logical sector 12)
get_next_sector:
        MOV     AX,CS:[data_at cum_sects]           ; logical sector number 12
        CALL    calc_sector_num                     ; calc C-H-N
        MOV     AL,BYTE PTR CS:[data_at sect_per_trk] ; = 9
        SUB     AL,CS:[data_at sector]              ; holds physical sector no.
        INC     AL                                  ; remaining sectors on track
        XOR     AH,AH
        PUSH    AX
        MOV     AH,02                               ; int 13h read (al) sectors
        CALL    read_sector
        POP     AX
        JB      boot_failure                        ; read error if CY
        SUB     CS:[data_at max_secs_ibmbio],AL     ; read no more than ibmbio length
        JBE     done_ibmbio_read                    ; no more to read
        ADD     CS:[data_at cum_sects],AX           ; update cum sector count
        MUL     CS:[data_at bytes_per_sector]
        ADD     BX,AX                               ; update es:bx read buffer
        JMP     get_next_sector

done_ibmbio_read:
        PUSH    CS
        POP     DS                      ; reset DS seg register
; ----- check if b: floppy drive is installed
        INT     11h                     ; equipment check
        ROL     AL,1                    ; get bits 6-7 into 0-1
        ROL     AL,1
        AND     AX,0003h                ; get the number of floppy drives installed
        JNZ     drive_count_ok          ; 0=1, 1=2, 2=3, 3=4
        INC     AX                      ; fake the b: drive
drive_count_ok:
        INC     AX                      ; bump up to get correct number
        MOV     CX,AX
        TEST    data_at drive,80h       ; is boot drive a hard disk?
        JNZ     jump_to_bio
        XOR     AX,AX                   ; no
jump_to_bio:
        MOV     BX,DS:[data_at sector_to_read]
;       JMP     0070H:0000H             ; this jumps to ibmbio entry
        DB      0eah,0,0,70h,0          ;NO WAY TO ASSEMBLE ABOVE

boot_failure:
        MOV     SI,location_of boot_err_mess
        CALL    print_line
freeze:
        JMP     freeze                  ; THIS FREEZES THINGS UP

; ------------------------------------------------------------
print_line:
;
; This routine prints a message on the screen
; On entry,
;       si      ptr to location of null-terminated string
;
        LODS    BYTE PTR CS:[SI]
        AND     AL,7fh
        JZ      b_return
        MOV     AH,0eh
        MOV     BX,0007h
        INT     10h
        JMP     print_line

; -------------------------------------------------------------------
check_sys:
;
; check for presence of ibmbio and ibmcom
;
        MOV     AX,0050h                ; segment for read sector buffer ..
        MOV     ES,AX                   ; .. for read into ES:BX = 0050:0000
        PUSH    CS
        POP     DS
        MOV     AX,CS:[data_at track_word]      ; holds logical dir sector (=5)
        CALL    calc_sector_num                 ; calc actual C-H-N
        MOV     BX,0000h
        MOV     AX,0201h                ; int 13h read 1 sector
        CALL    read_sector             ; get the dir
        JB      wait_key                ; did it work?
        XOR     DI,DI                   ; set index to 0
        MOV     CX,000bh                ; get ready to fix 11 bytes

; ----- compare the first two directory entries with ibmbio/ibmcom
lowercase:
        OR      BYTE PTR ES:[DI],20h        ; lower case 1st entry
        OR      BYTE PTR ES:[DI+20h],20h    ; lower case second entry
        INC     DI
        LOOP    lowercase                   ; keep it up 11 times
        XOR     DI,DI                       ; set first entry
        MOV     SI,location_of ibmbio       ; to be compared
        MOV     CX,000bh                    ; compare 11
        CLD
REPZ    CMPSB                            ; is it the same?
        JNZ     wait_key                 ; if no, emit msg and wait
        MOV     DI,0020h
        MOV     SI,location_of ibmdos    ; now ibmdos turn
        MOV     CX,000bh
REPZ    CMPSB
        JNZ     wait_key
b_return:
        RET

; ------------------------------------------------------------
wait_key:
;
; emit 'non-system disk' message and wait for keystroke ack
;
            MOV     SI,location_of non_sys_mess
            CALL    print_line
            MOV     AH,00h
            INT     16h
            STC
            RET

; -----------------------------------------------------------
calc_sector_num:
;
; Given a logical sector number, need physical C-H-N location (cyl-head-sec).
;       For example, the first directory entry on a 360KB DSDD 5.25"
;       diskette is at logical sector 5:
;
;       Logical   size in  physical
;       sector    sector   C-H-N        Area description
;
;       0         1        0-0-1        Boot sector (reserved sector)
;       1-2       2        0-0-2        FAT 1
;       3-4       2        0-0-4        FAT 2
;       5-11      7        0-0-6        Directory
;       12-719  708        0-1-3        Data area
;              ----
;       total   720         (40*9*2 = 720 sectors/diskette)
;
; on entry:     ax      logical sector number (0 = 0-0-1, 1 = 0-0-2, etc.)
;
; on exit:      ax      track   C:  track
;               dl      head    H:  head number (0 or 1)
;                       sector  N:  physical sector number (first sector on track = 1)
;
;
            PUSH    DS                  ; save old DS
            PUSH    CS                  ; establish addressability of cseg
            POP     DS
            XOR     DX,DX
            DIV     DS:[data_at sec_per_trk_word]
            INC     DL
            MOV     DS:[data_at sector],DL     ; physical sector number
            XOR     DX,DX                      ; clear register for divide result
            DIV     WORD PTR DS:[data_at sides]
            MOV     DS:[data_at head],DL       ; remainder gives head (0 or 1)
            MOV     data_at track,AX
            POP     DS                         ; restore DS
            RET

; --------------------------------------------------
read_sector:
;
; on entry:     ah      = 02            int 13h read sector function
;               al                      number of sectors to read
;               es:bx   0050:0000       read buffer
;               track
;               sector
;               head_drive              0000 = head 0, drive 0 (=A:)
;
            MOV     DX,CS:[data_at track]
            MOV     CL,06h
            SHL     DH,CL
            OR      DH,CS:[data_at sector]
            MOV     CX,DX
            XCHG    CH,CL
            MOV     DX,CS:[data_at head_drive]

; call int 13h with
;   ah = 02
;   al = number of sectors to read
;   ch = cylinder
;   cl = sector
;   dh = head
;   dl = drive, where 0=a:, 1=b:,... 80h=c:, 81=d:, ...
;   es:bx = seg:off of read buffer
            INT     13h
            RET

; --------------------------------------------
; miscellaneous data
;
sector_to_read          DW      0
non_sys_mess            DB     13,10
                        DB      'Non-System disk or disk error',13,10
                        DB      'Replace and strike any key when ready'
                        DB      13,10,0
boot_err_mess           DB      13,10
                        DB      'Disk Boot failure',13,10,0
ibmbio                  DB      'ibmbio  com0'
ibmdos                  DB      'ibmdos  com0'
                        DB      0,0,0,0,0,0,0
                        DB      55h,0AAh

main                    ENDP

cseg                    ENDS

                        END     enter


;  end of program 

