	 title	 KEYBMISC -- Miscellaneous Routines For KEYBECHO.COM
	 page	 58,122
	 name	 KEYBMISC

COMMENT|		Module Specifications

Environment:  IBM PC, tested under DOS 3.10 and 3.20.

Segmentation:
  Group PGROUP:
    Program segment PROG,  byte-aligned, public, class 'prog'
  Group DGROUP:
    Data    segment DATA,  byte-aligned, public, class 'data'

Link requirements:  None, standalone subroutine.

Program derived from:  None.

Original code by:  Bob Smith, May, 1986.

Modifications by:  None.

|
.xlist
	 include MASM.INC
	 include DOSCALL.INC
	 include KEYCALL.INC
	 include 8255.INC
	 include BIOSDATA.INC
.list

DGROUP	 group	 DATA

DATA	 segment byte public 'data' ; Start DATA segment

	 extrn	 INT16KEY:byte
	 extrn	 INT16KST:byte
	 extrn	 PRTSCFLG:byte
	 extrn	 SYSCALLFLG:byte

	 public  RID_LO,RID_HI,RCMD
RID_LO	 db	 0		; Low-order byte of Read ID byte
RID_HI	 db	 0		; High-order ... (must follow RID_LO)
RCMD	 db	 ?		; Command byte

	 public  LCL_FLG
LCL_FLG  dw	 0		; Local flags
@LCL_RID_LO equ  8000h		; Low-order Read ID byte
@LCL_RID_HI equ  4000h		; High-order Read ID byte
@LCL_RCMD equ	 2000h		; Command byte

DATA	 ends			; End DATA segment

PGROUP	 group	 PROG

PROG	 segment byte public 'prog' ; Start PROG segment
	 assume  cs:PGROUP,ds:DGROUP,es:nothing

	 public  DATA_SEG
DATA_SEG dw	 ?

	 public  OLDINT05_VEC,OLDINT05_OFF,OLDINT05_SEG
OLDINT05_VEC label dword	; Save area for old INT 05h handler
OLDINT05_OFF dw  ?
OLDINT05_SEG dw  ?

	 public  OLDINT15_VEC,OLDINT15_OFF,OLDINT15_SEG
OLDINT15_VEC label dword	; Save area for old INT 15h handler
OLDINT15_OFF dw  ?
OLDINT15_SEG dw  ?

	 public  OLDINT16_VEC,OLDINT16_OFF,OLDINT16_SEG
OLDINT16_VEC label dword	; Save area for old INT 16h handler
OLDINT16_OFF dw  ?
OLDINT16_SEG dw  ?

	 public  OLDINT1B_VEC,OLDINT1B_OFF,OLDINT1B_SEG
OLDINT1B_VEC label dword	; Save area for old INT 1Bh handler
OLDINT1B_OFF dw  ?
OLDINT1B_SEG dw  ?

COMMENT!=============================================================
	 NPPROC  CHECK_ENH_KEYB -- Check For Enhanced Keyboard
	 assume  ds:DGROUP,es:nothing
COMMENT|

Check for enhanced keyboard by testing bit @KBX in KB_FLAG_3
in the BIOS data area.	Because this technique relies upon
a specific bit definition within the BIOS data area, it might
not work on non-IBM systems, or for that matter on future
IBM systems.

On exit:

AX	 =	 1 if we're running on an enhanced keyboard
	 =	 0 if not

|

	 REGSAVE <ds>		; Save register

	 mov	 ax,seg BIOSDATA ; Get segment of BIOS data area
	 mov	 ds,ax		; DS ==> BIOSDATA
	 assume  ds:BIOSDATA	; Tell the assembler about it

	 xor	 ax,ax		; Assume not enhanced keyboard

	 test	 KB_FLAG_3,@KBX ; Check for enhanced keyboard
	 jz	 CHECK_ENH_KEYB_EXIT ; Not this time

	 mov	 ax,1		; It's the enhanced one
CHECK_ENH_KEYB_EXIT:
	 REGREST <ds>		; Restore
	 assume  ds:DGROUP	; Tell the assembler about it

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

CHECK_ENH_KEYB endp		; End CHECK_ENH_KEYB procedure
====================================================================!
	 NPPROC  CHECK_ENH_KEYB -- Check For Enhanced Keyboard
	 assume  ds:DGROUP,es:nothing
COMMENT|

Check for enhanced keyboard by attempting a Read Keyboard ID
function.  If we get a response, the enhanced keyboard is present;
otherwise, it's not.

On exit:

AX	 =	 Keyboard ID if we're running on an enhanced keyboard
	 =	 0 if not

|

	 pushf			; Save flags
	 cli			; Nobody move

	 mov	 al,@S2C_DIS	; Disable the keyboard interface
	 call	 SEND8042	; Send to keyboard controller
				; Ignore return code
	 call	 GETRCMD	; Read command byte from controller
	 call	 GETRID 	; Read ID byte from keyboard

	 mov	 al,@S2C_ENA	; Enable the keyboard interface
	 call	 SEND8042	; Send to keyboard controller
				; Ignore return code
	 mov	 ax,word ptr RID_LO ; Return to caller
	 popf			; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

CHECK_ENH_KEYB endp		; End CHECK_ENH_KEYB procedure
	 NPPROC  GETRCMD -- Get Command Byte
	 assume  ds:DGROUP,es:nothing
COMMENT|

Read the command byte

|

	 REGSAVE <ax>		; Save register

	 cli			; Nobody move

	 call	 CLEAROBUF	; Wait for the output buffer to clear

	 mov	 al,@S2C_RCMD	; Read the controller's command byte
	 call	 SEND8042	; Send to keyboard controller
	 jc	 ERR_RCMD	; Something went wrong

	 call	 WAITRESP	; Wait for a response, returned in AL
	 jc	 ERR_RCMD	; Something went wrong

	 mov	 RCMD,al	; Save response
	 or	 LCL_FLG,@LCL_RCMD ; Mark as present
ERR_RCMD:
	 REGREST <ax>		; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

GETRCMD  endp			; End GETRCMD procedure
	 NPPROC  GETRID -- Read Keyboard ID
	 assume  ds:DGROUP,es:nothing
COMMENT|

Read keyboard ID.

This value is returned in scan code format.  Thus to obtain the
actual value we need to turn off the controller's scan code
translation.

|

	 REGSAVE <ax>		; Save registers

	 cli			; Nobody move

	 call	 TRANSOFF	; Turn off scan code translation
	 jc	 ERR_RID2	; Something went wrong

	 mov	 al,@S2K_RID	; Function code to read ID
	 call	 SEND6805	; Send to keyboard, response in AL
	 jc	 ERR_RID1	; Jump if controller not responding

	 cmp	 al,@K2S_ACK	; Izit an ACK?
	 jne	 ERR_RID1	; Something went wrong
GETRID1:
	 call	 WAITRESP	; Wait for a response, returned in AL
	 jc	 ERR_RID1	; Something went wrong

	 cmp	 al,0ABh	; Correct response?
	 jne	 GETRID1	; No

	 mov	 RID_LO,al	; Save response
	 or	 LCL_FLG,@LCL_RID_LO ; Indicate we read it

	 call	 WAITRESP	; Wait for a response, returned in AL
	 jc	 ERR_RID1	; Something went wrong

	 mov	 RID_HI,al	; Save response
	 or	 LCL_FLG,@LCL_RID_HI ; Mark as present
ERR_RID1:
	 call	 RESTCMD	; Restore original command byte
				; Ignore return code
ERR_RID2:
	 REGREST <ax>		; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

GETRID	 endp			; End GETRID procedure
	 NPPROC  TRANSOFF -- Turn Off Scan Code Translation
	 assume  ds:DGROUP,es:nothing
COMMENT|

Turn off scan code translation.

On exit:

CF	 =	 0 if all went well
	 =	 1 otherwise

|

	 REGSAVE <ax>		; Save register

	 test	 LCL_FLG,@LCL_RCMD ; Is RCMD valid?
	 stc			; Assume not
	 jz	 TRANSOFF_EXIT	; No

	 mov	 al,@S2C_WCMD	; Write command byte
	 call	 SEND8042	; Send to keyboard controller
	 jc	 TRANSOFF_EXIT	; Something went wrong

	 call	 CLEARIBUF	; Wait for input buffer to clear
	 jc	 TRANSOFF_EXIT	; Error, controller not reading data

; Turn off scan code translation

	 mov	 al,RCMD	; Get the original command byte
	 and	 al,not mask $CMD_CPAT
	 or	 al,mask $CMD_DIS
	 out	 @8255_A,al	; Issue the command
TRANSOFF_EXIT:
	 REGREST <ax>		; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

TRANSOFF endp			; End TRANSOFF procedure
	 NPPROC  RESTCMD -- Restore Command Byte
	 assume  ds:DGROUP,es:nothing
COMMENT|

Restore command byte from RCMD.

On exit:

CF	 =	 0 if all went well
	 =	 1 otherwise

|

	 REGSAVE <ax>		; Save registers

	 mov	 al,@S2C_WCMD	; Write command byte
	 call	 SEND8042	; Send to keyboard controller
	 jc	 RESTCMD_EXIT	; Something went wrong

	 call	 CLEARIBUF	; Wait for input buffer to clear
	 jc	 RESTCMD_EXIT	; Error, controller not reading data

	 mov	 al,RCMD	; Get the original command byte
	 out	 @8255_A,al	; Issue the command
RESTCMD_EXIT:
	 REGREST <ax>		; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

RESTCMD  endp			; End RESTCMD procedure
	 NPPROC  WAITRESP -- Wait For A Response
	 assume  ds:DGROUP,es:nothing
COMMENT|

Wait a response from the keyboard or its controller.

On exit:

CF	 =	 1 if no response
	 =	 0 otherwise

AL	 =	 response if CF=0
AH	 =	 clobbered

|

	 REGSAVE <bx,cx>	; Save registers

	 clc			; Assume all goes well
	 lahf			; Load AH with flags
	 pushf			; Save flags
	 cli			; Nobody move

; Wait for a response

	 mov	 bl,12		; Outer loop counter
				; (just to be safe on 16MHz 386)
	 xor	 cx,cx		; Inner loop counter
RESP1:
	 in	 al,@8042_ST	; Get status from keyboard

	 and	 al,mask $OUTFULL ; Check Output Buffer Full flag
	 loopz	 RESP1		; Jump if no response as yet
	 jz	 RESP2		; Still no response

	 jmp	 $+2		; Take into account I/O delay

	 in	 al,@8255_A	; Read in the code

	 jmp	 short RESP_EXIT ; Join common exit code
RESP2:
	 dec	 bl		; Try again
	 jnz	 RESP1		; Jump if more tries available

	 inc	 ah		; Set CF in AH
RESP_EXIT:
	 POPFF			; Restore flags with caller's IF
	 sahf			; Store AH into flags

	 REGREST <cx,bx>	; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

WAITRESP endp			; End WAITRESP procedure
	 NPPROC  SEND8042 -- Send Command To Keyboard Controller
	 assume  ds:DGROUP,es:nothing
COMMENT|

Send the command in AL to the keyboard controller port 64h.
There is no acknowledgement of this command.

On exit:

CF	 =	 1 if keyboard controller not responding
	 =	 0 otherwise

|

	 REGSAVE <ax>		; Save for a moment

	 clc			; Assume all goes well
	 lahf			; Load AH with flags
	 pushf			; Save flags
	 cli			; Nobody move

	 call	 CLEARIBUF	; Wait for input buffer to clear
	 jc	 CTRL_EXIT	; Jump if controller not responding

	 out	 @8042_ST,al	; Send the command
CTRL_EXIT:
	 adc	 ah,0		; Set CF in AH as necessary
	 POPFF			; Restore flags with caller's IF
	 sahf			; Store AH into flags

	 REGREST <ax>		; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

SEND8042 endp			; End SEND8042 procedure
	 NPPROC  SEND6805 -- Send Data in AL to Keyboard
	 assume  ds:DGROUP,es:nothing
COMMENT|

Send AL to keyboard.

On exit:

CF	 =	 1 if timeout
	 =	 0 otherwise

AL	 =	 keyboard response if CF=0

AH is clobbered.

|

	 REGSAVE <cx>		; Save for a moment

	 mov	 cx,6		; # retries of resend

	 clc			; Assume all goes well
	 lahf			; Load AH with flags
	 pushf			; Save flags
	 cli			; Nobody move
SEND0:
	 REGSAVE <ax>		; Save for a moment

	 call	 CLEARIBUF	; Wait for input buffer to clear
	 jc	 SEND1		; Error, controller not reading data
				; (note CF=1)
	 out	 @8255_A,al	; Issue the command

	 call	 WAITRESP	; Wait for a response, returned in AL
	 jc	 SEND1		; Something went wrong

	 cmp	 al,@K2S_RESEND ; Izit a resend?
	 clc			; In case it isn't
	 jne	 SEND1		; No

	 REGREST <ax>		; Restore

	 loop	 SEND0		; Jump if more retries

	 stc			; Indicate something went wrong

	 REGSAVE <ax>		; Save for a moment
SEND1:
	 inc	 sp		; Restore stack pointer
	 inc	 sp		; ...without changing CF

	 adc	 ah,0		; Set CF in AH as necessary
	 POPFF			; Restore flags with caller's IF
	 sahf			; Store AH into flags

	 REGREST <cx>		; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

SEND6805 endp			; End SEND6805 procedure
	 NPPROC  CLEARIBUF -- Wait For The Input Buffer To Clear
	 assume  ds:DGROUP,es:nothing
COMMENT|

Wait for the one-byte input buffer to clear.

On exit:

CF	 =	 0 if buffer empty
	 =	 1 otherwise

|


	 REGSAVE <ax,cx>	; Save registers

	 clc			; Assume all goes well
	 lahf			; Load AH with flags
	 pushf			; Save flags
	 cli			; Nobody move

	 xor	 cx,cx		; Loop counter
CLEARIBUF1:
	 in	 al,@8042_ST	; Get status from keyboard

	 and	 al,mask $INPFULL ; Check Input Buffer Full flag
	 loopnz  CLEARIBUF1	; Last char not read as yet
	 jz	 CLEARIBUF_EXIT ; Jump if buffer clear before then
				; (note CF=0)
	 stc			; Mark as in error
CLEARIBUF_EXIT:
	 adc	 ah,0		; Set CF in AH as necessary
	 POPFF			; Restore flags with caller's IF
	 sahf			; Store AH into flags

	 REGREST <cx,ax>	; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

CLEARIBUF endp			; End CLEARIBUF procedure
	 NPPROC  CLEAROBUF -- Wait For The Output Buffer To Clear
	 assume  ds:DGROUP,es:nothing
COMMENT|

Wait for the one-byte input buffer to clear.

|


	 REGSAVE <ax>		; Save register

	 cli			; Nobody move
CLEAROBUF1:
	 in	 al,@8042_ST	; Get status from keyboard

	 and	 al,mask $OUTFULL ; Check Output Buffer Full flag
	 jz	 CLEAROBUF_EXIT ; Jump if buffer clear before

	 jmp	 $+2		; I/O delay

	 in	 al,@8255_A	; Purge the character

	 jmp	 CLEAROBUF1	; Continue on
CLEAROBUF_EXIT:
	 REGREST <ax>		; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

CLEAROBUF endp			; End CLEAROBUF procedure
	 NPPROC  GETKEY -- Get A Keystroke
	 assume  ds:DGROUP,es:nothing

GETKEY1:
	 xor	 ax,ax		; Just so we have something to return

	 cmp	 PRTSCFLG,0	; Check for PrtSc
	 jne	 GETKEY_EXIT	; Join common exit code

	 cmp	 SYSCALLFLG,0	; Check for SysReq
	 jne	 GETKEY_EXIT	; Join common exit code

	 mov	 ah,INT16KST	; Get key status function code
	 KEYCALL		; Request keyboard service
	 jz	 GETKEY1	; Jump if nothing available as yet

	 mov	 ah,INT16KEY	; Get read key function code
	 KEYCALL		; Request keyboard service
GETKEY_EXIT:
	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

GETKEY	 endp			; End GETKEY procedure
	 FPPROC  INT16 -- Keyboard I/O Handler
	 assume  ds:nothing,es:nothing

	 pushf			; Save flags

	 sti			; Allow interrupts

	 cmp	 ah,01h 	; Check for functions 00h-01h
	 ja	 INT16_ORIG	; Too big for us

	 or	 ah,10h 	; Convert to extended functions
INT16_ORIG:
	 popf			; Restore flags

	 jmp	 OLDINT16_VEC	; Continue on

	 assume  ds:nothing,es:nothing

INT16	 endp			; End INT16 procedure
	 NPPROC  INST16 -- Install INT 16h Handler
	 assume  ds:DGROUP,es:nothing

	 REGSAVE <ax,bx,dx>	; Save registers

	 push	 es		; Save for a moment
	 mov	 al,16h 	; Intercept this one
	 DOSCALL @GETINT	; Return old address in ES:BX
	 assume  es:nothing

	 mov	 OLDINT16_OFF,bx ; Save to restore later
	 mov	 OLDINT16_SEG,es
	 pop	 es		; Restore
	 assume  es:nothing

	 push	 ds		; Save for a moment

	 push	 cs		; Setup DS for SETINT
	 pop	 ds
	 assume  ds:PGROUP	; Tell the assembler

	 mov	 al,16h 	; Intercept this one
	 DOSCALL @SETINT,ds:INT16 ; Install our own handler

	 pop	 ds
	 assume  ds:DGROUP	; Tell the assembler

	 REGREST <dx,bx,ax>	; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

INST16	 endp			; End INST16 procedure
	 NPPROC  REST16 -- Restore INT 16h Handler
	 assume  ds:DGROUP,es:nothing

	 REGSAVE <ax,dx>	; Save registers

	 push	 ds		; Save for a moment

	 lds	 dx,OLDINT16_VEC ; DS:DX ==> old handler
	 assume  ds:nothing	; Tell the assembler

	 mov	 al,16h 	; Restore this one
	 DOSCALL @SETINT

	 pop	 ds		; Restore
	 assume  ds:DGROUP

	 REGREST <dx,ax>	; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

REST16	 endp			; End REST16 procedure
	 FPPROC  INT05 -- Print Screen Handler
	 assume  ds:nothing,es:nothing

	 sti			; Allow interrupts

	 push	 ds		; Save for a moment

	 mov	 ds,DATA_SEG	; Address data segment
	 assume  ds:DGROUP	; Tell the assembler
	 mov	 PRTSCFLG,1	; Tell KEYBECHO.COM we've been called

	 pop	 ds		; Restore
	 assume  ds:nothing

	 iret			; Return to caller

	 assume  ds:nothing,es:nothing

INT05	 endp			; End INT05 procedure
	 FPPROC  INT15 -- System Call Handler
	 assume  ds:nothing,es:nothing

	 pushf			; Save flags

	 sti			; Allow interrupts

	 cmp	 ax,8500h	; System key make?
	 jne	 INT15_ORIG	; Not this time

	 push	 ds		; Save for a moment

	 mov	 ds,DATA_SEG	; Address data segment
	 assume  ds:DGROUP	; Tell the assembler
	 mov	 SYSCALLFLG,1	; Tell KEYBECHO.COM we've been called

	 pop	 ds		; Restore
	 assume  ds:nothing
INT15_ORIG:
	 popf			; Restore flags

	 jmp	 OLDINT15_VEC	; Continue on

	 assume  ds:nothing,es:nothing

INT15	 endp			; End INT15 procedure
	 FPPROC  INT1B -- Control-Break Handler
	 assume  ds:nothing,es:nothing

	 iret			; Return to caller

	 assume  ds:nothing,es:nothing

INT1B	 endp			; End INT1B procedure
	 NPPROC  INSTINTS -- Install Interrupt Handlers
	 assume  ds:DGROUP,es:nothing

	 REGSAVE <ax,bx,dx>	; Save registers

	 mov	 DATA_SEG,ds	; Save for INT05 later

	 push	 es		; Save for a moment

	 mov	 al,05h 	; Intercept this one
	 DOSCALL @GETINT	; Return old address in ES:BX
	 assume  es:nothing

	 mov	 OLDINT05_OFF,bx ; Save to restore later
	 mov	 OLDINT05_SEG,es

	 mov	 al,15h 	; Intercept this one
	 DOSCALL @GETINT	; Return old address in ES:BX
	 assume  es:nothing

	 mov	 OLDINT15_OFF,bx ; Save to restore later
	 mov	 OLDINT15_SEG,es

	 mov	 al,1Bh 	; Intercept this one
	 DOSCALL @GETINT	; Return old address in ES:BX
	 assume  es:nothing

	 mov	 OLDINT1B_OFF,bx ; Save to restore later
	 mov	 OLDINT1B_SEG,es

	 pop	 es		; Restore
	 assume  es:nothing

	 push	 ds		; Save for a moment

	 push	 cs		; Setup DS for SETINT
	 pop	 ds
	 assume  ds:PGROUP	; Tell the assembler

	 mov	 al,05h 	; Intercept this one
	 DOSCALL @SETINT,ds:INT05 ; Install our own handler

	 mov	 al,15h 	; Intercept this one
	 DOSCALL @SETINT,ds:INT15 ; Install our own handler

	 mov	 al,1Bh 	; Intercept this one
	 DOSCALL @SETINT,ds:INT1B ; Install our own handler

	 pop	 ds
	 assume  ds:DGROUP	; Tell the assembler

	 REGREST <dx,bx,ax>	; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

INSTINTS endp			; End INSTINTS procedure
	 NPPROC  RESTINTS -- Restore Interrupt Handlers
	 assume  ds:DGROUP,es:nothing

	 REGSAVE <ax,dx>	; Save registers

	 push	 ds		; Save for a moment

	 lds	 dx,OLDINT05_VEC ; DS:DX ==> old handler
	 assume  ds:nothing	; Tell the assembler
	 mov	 al,05h 	; Restore this one
	 DOSCALL @SETINT

	 lds	 dx,OLDINT15_VEC ; DS:DX ==> old handler
	 assume  ds:nothing	; Tell the assembler
	 mov	 al,15h 	; Restore this one
	 DOSCALL @SETINT

	 lds	 dx,OLDINT1B_VEC ; DS:DX ==> old handler
	 assume  ds:nothing	; Tell the assembler
	 mov	 al,1Bh 	; Restore this one
	 DOSCALL @SETINT

	 pop	 ds		; Restore
	 assume  ds:DGROUP

	 REGREST <dx,ax>	; Restore

	 ret			; Return to caller

	 assume  ds:nothing,es:nothing

RESTINTS endp			; End RESTINTS procedure

PROG	 ends			; End PROG segment

	 MEND			; End KEYBMISC module
