/****************************************************************************
*
*						MegaGraph Graphics Library
*
*                   Copyright (C) 1993 Kendall Bennett.
*							All rights reserved.
*
* Filename:		$RCSfile$
* Version:		$Revision$
*
* Language:		ANSI C
* Environment:	IBM PC (MS DOS)
*
* Description:	Routine to determine the available video modes for a
*				specified video card, given memory restrictions.
*
* $Id$
*
* Revision History:
* -----------------
*
* $Log$
****************************************************************************/

#include "mgraph.h"

/*--------------------------- Global Variables ----------------------------*/

int		__modeTable[grMAXMODE+1];	/* Internal table of video modes	*/

/*------------------------- Implementation --------------------------------*/

/* Routines in 'detectgr.asm' for returning the list of possible video
 * modes for this adapter and display pages.
 */

void _getSuperVGAinfo(char far * far *modes,int far *memory);
int _getSuperVGAPages(int mode);

PRIVATE int loadVGAModes(int i)
/****************************************************************************
*
* Function:		loadVGAModes
* Parameters:	i	- Index to start loading modes at
* Returns:		Index of next available space in mode table.
*
* Description:	Fills in all of the standard VGA video modes in the mode
*				table.
*
****************************************************************************/
{
	__modeTable[i++] = grEGA_320x200x16;
	__modeTable[i++] = grEGA_640x200x16;
	__modeTable[i++] = grEGA_640x350x16;
	__modeTable[i++] = grVGA_640x400x16;
	__modeTable[i++] = grVGA_640x480x16;
	__modeTable[i++] = grVGA_320x200x256;
	__modeTable[i++] = grVGAX_320x200x256;
	__modeTable[i++] = grVGAX_320x240x256;
	__modeTable[i++] = grVGAX_320x400x256;
	__modeTable[i++] = grVGAX_320x480x256;
	__modeTable[i++] = grVGAX_360x200x256;
	__modeTable[i++] = grVGAX_360x240x256;
	__modeTable[i++] = grVGAX_360x400x256;
	__modeTable[i++] = grVGAX_360x480x256;
	return i;
}

PUBLIC int * MGL_availableModes(int driver,int *memory)
/****************************************************************************
*
* Function:		MGL_availableModes
* Parameters:	driver	- Graphics device driver to check.
*				memory	- Place to store amount of memory on video card.
*
* Returns:		Returns a pointer to a list of all the video modes that are
*				currently available. This routine can be called before
*				MGL_init() is called to determine the video modes available.
*				The list of video modes is terminated by a -1, but you
*				will have to call MGL_detectGraph() to determine the
*				currently installed video card.
*
****************************************************************************/
{
	int		i;
	char	far *modes;

	switch(driver) {
		case grMCGA:
			__modeTable[0] = grVGA_320x200x256;
			__modeTable[1] = -1;		/* Terminate the table	*/
			*memory = 64;
			break;
		case grEGA:
			__modeTable[0] = grEGA_320x200x16;
			__modeTable[1] = grEGA_640x200x16;
			__modeTable[2] = grEGA_640x350x16;
			__modeTable[3] = -1;		/* Terminate the table	*/
			*memory = 256;
			break;
		case grVGA:
			i = loadVGAModes(0);
			__modeTable[i] = -1;		/* Terminate the table	*/
			*memory = 256;
			break;
		default:
			if (driver >= __FIRST_SVGA && driver <= __LAST_SVGA) {
				/* We have a Super VGA video card. We know that we can use
				 * all of the standard VGA video modes, but we must also
				 * determine what SuperVGA video modes are available. We
				 * obtain this information from the DetectGraph module.
				 */

				i = loadVGAModes(0);

				/* Install all detected SuperVGA modes, filtering
				 * those not supported with the memory installed
				 */

				_getSuperVGAinfo(&modes,memory);
				while (*modes != -1) {
					if (*memory >= 4096)	/* Fix strange memory sizes	*/
						*memory = 4096;
					else if (*memory >= 3072)
						*memory = 3072;
					else if (*memory >= 2048)
						*memory = 2048;
					else if (*memory >= 1024)
						*memory = 1024;
					else if (*memory >= 512)
						*memory = 512;
					else if (*memory >= 256)
						*memory = 256;

					switch (*memory) {
						case 4096:
							switch (*modes) {
								case grSVGA_1280x1024x32k:
								case grSVGA_1024x768x16m:
								case grSVGA_1280x1024x16m:
									__modeTable[i++] = *modes;
									break;
								}
						case 3072:
							// This needs to be filled in!!!!!
						case 2048:
							switch (*modes) {
								case grSVGA_1280x1024x256:
								case grSVGA_1024x768x32k:
								case grSVGA_800x600x16m:
									__modeTable[i++] = *modes;
									break;
								}
						case 1024:
							switch (*modes) {
								case grSVGA_1280x1024x16:
								case grSVGA_1024x768x256:
								case grSVGA_640x480x32k:
								case grSVGA_800x600x32k:
								case grSVGA_640x350x16m:
								case grSVGA_640x400x16m:
								case grSVGA_640x480x16m:
									__modeTable[i++] = *modes;
									break;
								}
						case 512:
							switch (*modes) {
								case grSVGA_1024x768x16:
								case grSVGA_640x480x256:
								case grSVGA_800x600x256:
								case grSVGA_640x350x32k:
								case grSVGA_640x400x32k:
									__modeTable[i++] = *modes;
									break;
								}
						case 256:
							switch (*modes) {
								case grSVGA_800x600x16:
								case grSVGA_640x350x256:
								case grSVGA_640x400x256:
								case grSVGA_320x200x32k:
								case grSVGA_320x200x16m:
									__modeTable[i++] = *modes;
									break;
								}
							break;
						}
					modes++;
					}
				__modeTable[i] = -1;		/* Terminate the table	*/
				}
			else {
				/* No video modes are supported for this adapter */

				__modeTable[0] = -1;		/* Terminate the table	*/
				}
		}
	return __modeTable;
}

PUBLIC int MGL_availablePages(int driver,int mode)
/****************************************************************************
*
* Function:		MGL_availablePages
* Parameters:	driver	- Graphics device driver to check.
*				mode	- Video mode to check
* Returns:		Number of available video pages for the mode, -1 on error.
*
* Description:	Determines the number of available video pages for a
*				specific video mode and driver. This routine assumes that
*				the MGL_detectGraph() routine has already been called
*				to obtain the video device driver ID.
*
*				If the video mode specified is invalid, a -1 is returned.
*
****************************************************************************/
{
	int		memory,*modes;

	/* Get list of available video modes and check that the mode specified
	 * is currently valid
	 */

	modes = MGL_availableModes(driver,&memory);

	while (*modes != -1) {
		if (mode == *modes)
			break;
		modes++;
		}
	if (*modes == -1)
		return -1;

	switch(driver) {
		case grMCGA:
			return 1;
		case grEGA:
		case grVGA:
			switch (mode) {
				case grEGA_320x200x16:
					return 8;
				case grEGA_640x200x16:
					return 4;
				case grEGA_640x350x16:
					return 2;
				case grVGA_640x400x16:
					return 2;
				case grVGA_640x480x16:
					return 1;
				}
			return -1;
		default:
			if (driver >= __FIRST_SVGA && driver <= __LAST_SVGA) {
				/* We have a Super VGA video card. We simply call the
				 * assembly language routine to compute the number
				 * of video pages.
				 */

				return _getSuperVGAPages(mode);
				}
			else {
				/* No video modes are supported for this adapter */

				return -1;
				}
		}
}
