/****************************************************************************
*
*							  The MegaToolbox
*
*					Copyright (C) 1991 Kendall Bennett.
*							All rights reserved.
*
* Filename:		$RCSfile: mouse.h $
* Version:		$Revision: 1.4 $
*
* Language:		ANSI C
* Environment:	any
*
* Description:	Header file for the mouse interface module.
*
*	For full compatibility with this module, you must have one of the
*	following mice:
*
*		-	Microsoft Mouse version 6.1 or later, or any mouse compatible
*			with this mouse.
*		-	Logitech Mouse version 3.4 or later.
*		-	Mouse Systems' PC Mouse version 6.22 or later.
*		-	IMSI mouse version 6.11 or later.
*
* $Id: mouse.h 1.4 1992/01/28 20:42:57 kjb release $
*
* Revision History:
* -----------------
*
* $Log: mouse.h $
* Revision 1.4  1992/01/28  20:42:57  kjb
* Converted to a memory model independant library.
*
* Revision 1.3  92/01/06  18:57:25  kjb
* Removed support for the condifional off routine.
* 
* Revision 1.2  92/01/01  15:50:31  kjb
* Factorised into a completely independant module.
* 
* Revision 1.1  91/11/28  01:57:47  kjb
* Initial revision
* 
****************************************************************************/

#ifndef	__MOUSE_H
#define	__MOUSE_H

#ifndef __DEBUG_H
#include "debug.h"
#endif

/*--------------------- Macros and type definitions -----------------------*/

#define	call_mouse()	geninterrupt(0x33)

#define leftBut		0			/* Button types for MS_pressed() etc	*/
#define	rightBut	1
#define middleBut	2

#define	SOFTWARE	0			/* Text cursor types					*/
#define HARDWARE	1

typedef struct {				/* Mouse status structure				*/
	int			but_status;		/* Which buttons are currently down		*/
	int			but_count;		/* Number of presses for button			*/
	int			x;				/* Position of last button press h		*/
	int			y;				/* Position of last button press v		*/
	} ms_status;

typedef struct {				/* Interrupt subroutine info structure	*/
	void	far	*addr;			/* Address of interrupt subroutine		*/
	int			mask;			/* Mask for subroutine					*/
	} intsub;

typedef	unsigned int bits16[16];

typedef	struct {				/* Graphical cursor structure			*/
	bits16		screenmask;
	bits16		cursormask;
	int			hotspot_x;
	int			hotspot_y;
	} cursor;

/*------------------------- Function Prototypes ---------------------------*/

#ifdef	__cplusplus
extern "C" {			/* Use "C" linkage when in C++ mode	*/
#endif

#ifdef	__BUILDING_MOUSE__
#define	__FAR	huge
#else
#define	__FAR	far
#endif

int		__FAR MS_init(void);
void	__FAR MS_exit(void);
void	__FAR MS_suspend(void);
void	__FAR MS_resume(void);
int		__FAR MS_hardReset(void);
void	__FAR MS_show(void);
void	__FAR MS_hide(void);
void	__FAR MS_pos(ms_status far *stat);
void	__FAR MS_moveTo(int h,int v);
void	__FAR MS_pressed(int which,ms_status far *stat);
void	__FAR MS_released(int which,ms_status far *stat);
void	__FAR MS_constrain(int left,int top,int right,int bottom);
void	__FAR MS_setCursor(cursor far *curs);
void	__FAR MS_setTextCursor(int type,int screenMask,int cursorMask);
void	__FAR MS_motion(int far *h,int far *v);
void	__FAR MS_speed(int hratio,int vratio);
void	__FAR MS_dblSpeed(int threshold);
void 	__FAR MS_swapIntSub(int num,intsub far *newSub,intsub far *oldSub);
int 	__FAR MS_stateSize(void);
void	__FAR MS_saveState(void far *theState);
void	__FAR MS_restoreState(void far *theState);
void	__FAR MS_setCrtPage(int page);
int		__FAR MS_getCrtPage(void);
int		__FAR MS_softReset(void);

#ifdef	__cplusplus
}						/* End of "C" linkage for C++	*/
#endif

#undef	__FAR

#endif  /* __MOUSE_H */
