/****************************************************************************
*
*						   SuperVGA Test Library
*
*                   Copyright (C) 1993 Kendall Bennett.
*							All rights reserved.
*
* Filename:		$RCSfile$
* Version:		$Revision$
*
* Language:		ANSI C
* Environment:	IBM PC (MSDOS)
*
* Description:	Simple program to test the operation of the SuperVGA
*				bank switching code and page flipping code for the
*				16 color SuperVGA video modes.
*
*				MUST be compiled in the small model.
*
* $Id$
*
* Revision History:
* -----------------
*
* $Log$
****************************************************************************/

#pragma	inline

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <conio.h>
#include "mgraph.h"

/*-------------------------- Function Prototypes --------------------------*/

/* In detectgr.asm */

int _getDACType(void);

/* In svga16.asm */

int initSuperVGA(int driver,int chipID,int mode,int memory);
int setSuperVGAMode(void);
void restoreMode(void);
void getVideoInfo(int *xres,int *yres,int *bytesperline,int *maxpage);
void putPixel(int x,int y,int color);
void copyTest(void);
void clear(void);
void setActivePage(int page);
void setVisualPage(int page);

/* Global variables */

int		_grResult;
int		pageFlipping = 0;
int		twoBanks = 0;
int		x,y,maxx,maxy,maxpage,bytesperline;
int		maxcolor = 15,defcolor = 15;
int		driver,chipID,mode,memory;
int		far *modeList;

/* The following are used by the MGL_detectGraph() routine */

int		_VESAFirst = false;			/* Check for VESA SuperVGA's first	*/
int		_ignoreSVGA = false;		/* Ignore SuperVGA adapters			*/

#include "modes.inc"

/*----------------------------- Implementation ----------------------------*/

void line(int x1,int y1,int x2,int y2,int color)
/****************************************************************************
*
* Function:		line
* Parameters:	x1,y1		- First endpoint of line
*				x2,y2		- Second endpoint of line
*
* Description:	Scan convert a line segment using the MidPoint Digital
*				Differential Analyser algorithm.
*
****************************************************************************/
{
	int		d;						/* Decision variable				*/
	int		dx,dy;					/* Dx and Dy values for the line	*/
	int		Eincr,NEincr;			/* Decision variable increments		*/
	int		yincr;					/* Increment for y values			*/
	int		t;						/* Counters etc.					*/

	dx = abs(x2 - x1);
	dy = abs(y2 - y1);

	if (dy <= dx) {

		/* We have a line with a slope between -1 and 1
		 *
		 * Ensure that we are always scan converting the line from left to
		 * right to ensure that we produce the same line from P1 to P0 as the
		 * line from P0 to P1.
		 */

		if (x2 < x1) {
			t = x2; x2 = x1; x1 = t;	/* Swap X coordinates			*/
			t = y2; y2 = y1; y1 = t;	/* Swap Y coordinates			*/
			}

		if (y2 > y1)
			yincr = 1;
		else
			yincr = -1;

		d = 2*dy - dx;				/* Initial decision variable value	*/
		Eincr = 2*dy;				/* Increment to move to E pixel		*/
		NEincr = 2*(dy - dx);		/* Increment to move to NE pixel	*/

		putPixel(x1,y1,color);		/* Draw the first point at (x1,y1)	*/

		/* Incrementally determine the positions of the remaining pixels
		 */

		for (x1++; x1 <= x2; x1++) {
			if (d < 0) {
				d += Eincr;			/* Choose the Eastern Pixel			*/
				}
			else {
				d += NEincr;		/* Choose the North Eastern Pixel	*/
				y1 += yincr;		/* (or SE pixel for dx/dy < 0!)		*/
				}
			putPixel(x1,y1,color);	/* Draw the point					*/
			}
		}
	else {

		/* We have a line with a slope between -1 and 1 (ie: includes
		 * vertical lines). We must swap our x and y coordinates for this.
		 *
		 * Ensure that we are always scan converting the line from left to
		 * right to ensure that we produce the same line from P1 to P0 as the
		 * line from P0 to P1.
		 */

		if (y2 < y1) {
			t = x2; x2 = x1; x1 = t;	/* Swap X coordinates			*/
			t = y2; y2 = y1; y1 = t;	/* Swap Y coordinates			*/
			}

		if (x2 > x1)
			yincr = 1;
		else
			yincr = -1;

		d = 2*dx - dy;				/* Initial decision variable value	*/
		Eincr = 2*dx;				/* Increment to move to E pixel		*/
		NEincr = 2*(dx - dy);		/* Increment to move to NE pixel	*/

		putPixel(x1,y1,color);		/* Draw the first point at (x1,y1)	*/

		/* Incrementally determine the positions of the remaining pixels
		 */

		for (y1++; y1 <= y2; y1++) {
			if (d < 0) {
				d += Eincr;			/* Choose the Eastern Pixel			*/
				}
			else {
				d += NEincr;		/* Choose the North Eastern Pixel	*/
				x1 += yincr;		/* (or SE pixel for dx/dy < 0!)		*/
				}
			putPixel(x1,y1,color);	/* Draw the point					*/
			}
		}
}

void writeText(int x,int y,uchar *str)
/****************************************************************************
*
* Function:		writeText
* Parameters:	x,y		- Position to begin drawing string at
*				str		- String to draw
*
* Description:	Draws a string using the BIOS 8x16 video font by plotting
*				each pixel in the characters individually. This should
*				work for all video modes.
*
****************************************************************************/
{
	uchar	byte;
	int		i,j,k,length,ch;
	uchar	far *_font;

	asm {				/* Bloody inline assembly - we need it here tho	*/
		push	bp
		mov		bh,0x06;
		mov		ax,0x1130;
		int		0x10;
		mov		ax,es;
		mov		bx,bp;
		pop		bp;
		mov		i,ax;
		mov		j,bx;
		}
	_font = FP(i,j);

	length = strlen(str);
	for (k = 0; k < length; k++) {
		ch = str[k];
		for (j = 0; j < 16; j++) {
			byte = *(_font + ch * 16 + j);
			for (i = 0; i < 8; i++) {
				if ((byte & 0x80) != 0)
					putPixel(x+i,y+j,defcolor);
				byte <<= 1;
				}
			}
		x += 8;
		}
}

void moireTest(void)
/****************************************************************************
*
* Function:		moireTest
*
* Description:	Draws a simple Moire pattern on the display screen using
*				lines, and waits for a key press.
*
****************************************************************************/
{
	uchar	buf[80];
	int		i;

	clear();
	for (i = 0; i < maxx; i += 10) {
		line(maxx/2,maxy/2,i,0,i % maxcolor);
		line(maxx/2,maxy/2,i,maxy,(i+1) % maxcolor);
		}
	for (i = 0; i < maxy; i += 10) {
		line(maxx/2,maxy/2,0,i,(i+2) % maxcolor);
		line(maxx/2,maxy/2,maxx,i,(i+3) % maxcolor);
		}

	if (maxx != 319) {
		x = 80;
		y = 80;
		writeText(x,y,"Bank switching test");	y += 32;
		sprintf(buf,"Video mode: %s",modeNames[mode]);
		writeText(x,y,buf);	y += 16;
		sprintf(buf,"Maximum x: %d, Maximum y: %d, BytesPerLine %d, Pages: %d",
			maxx,maxy,bytesperline,maxpage+1);
		writeText(x,y,buf);	y += 32;
		writeText(x,y,"You should see a colorful Moire pattern on the screen");
		y += 16;
		}
	else {
		x = 40;
		y = 40;
		}
	writeText(x,y,"Press any key to continue");
	y += 32;
	getch();
}

void readWriteTest(void)
/****************************************************************************
*
* Function:		readWriteTest
*
* Description:	Test the separate read/write bank routines if available.
*				We do this by copying the top half of video memory to
*				the bottom half of the second video page.
*
*				This test is designed to run only in the 800x600 video mode.
*
****************************************************************************/
{
	if (twoBanks && pageFlipping && maxpage != 0 && (maxx == 799)) {
		writeText(x,y,"To test the separate read/write banks, the top half of");
		y += 16;
		writeText(x,y,"this display page should be moved to the bottom half of");
		y += 16;
		writeText(x,y,"the second display page");
		setActivePage(1);
		clear();
		setVisualPage(1);
		copyTest();
		x = y = 80;
		writeText(x,y,"Press any key to continue");
		getch();
		}
}

void pageFlipTest(void)
/****************************************************************************
*
* Function:		pageFlipTest
*
* Description:	Animates a line on the display using page flipping if
*				page flipping is active.
*
****************************************************************************/
{
	int		i,j,istep,jstep,color,apage,vpage;
	char	buf[80];

	if (maxpage != 0) {
		vpage = 0;
		apage = 1;
		setActivePage(apage);
		setVisualPage(vpage);
		i = 0;
		j = maxy;
		istep = 2;
		jstep = -2;
		color = 15;
		while (!kbhit()) {
			setActivePage(apage);
			clear();
			sprintf(buf,"Page %d of %d", vpage+1, maxpage+1);
			if (maxx == 319) {
				writeText(0,80,"Page flipping - should be no flicker");
				writeText(0,100,buf);
				}
			else {
				writeText(80,80,"Page flipping - should be no flicker");
				writeText(80,100,buf);
				}
			line(i,0,maxx-i,maxy,color);
			line(0,maxy-j,maxx,j,color);
			vpage = ++vpage % (maxpage+1);
			setVisualPage(vpage);
			apage = ++apage % (maxpage+1);
			i += istep;
			if (i > maxx) {
				i = maxx-2;
				istep = -2;
				}
			if (i < 0)	i = istep = 2;
			j += jstep;
			if (j > maxy) {
				j = maxy-2;
				jstep = -2;
				}
			if (j < 0)	j = jstep = 2;
			}
		getch();				/* Swallow keypress */
		}
}

void testingComplete(void)
/****************************************************************************
*
* Function:		testingComplete
*
* Description:	Clears the first display page and puts up a message.
*
****************************************************************************/
{
	setActivePage(0);
	setVisualPage(0);
	clear();

	if (maxx == 319) {
		writeText(0,40,"Testing complete");
		writeText(0,60,"press any key to return to text mode");
		}
	else
		writeText(80,80,"Testing complete - press any key to return to text mode");
	getch();
}

void autoDetect(void)
/****************************************************************************
*
* Function:		autoDetect
*
* Description:	Auto detect the video adapter and video modes available,
*				and gathering information such as hardware capabilities.
*
****************************************************************************/
{
	int		ch;

	printf("Check for VESA BIOS extensions first (y/[n])? ");
	ch = getch();
	if (ch == 'y' || ch == 'Y')
		_VESAFirst = true;
	MGL_detectGraph(&driver,&chipID,&mode);
	modeList = MGL_availableModes(driver,&memory);

	/* Get info about hardware support for page flipping and two banks */

	ch = initSuperVGA(driver,chipID,grVGA_640x480x16,memory);
	pageFlipping = ch & 1;
	twoBanks = ch & 2;
}

void main(void)
{
	int		i,choice,maxmenu;
	int		menu[20];
	char	buf[80];

	autoDetect();

	while (true) {
		clrscr();
		printf("16 color SuperVGA test program\n\n");
		printf("Video Card: %s ",driverNames[driver]);
		if (driver > grSVGA && chipsetNames[driver-grSVGA-1])
			printf("(%s)",chipsetNames[driver-grSVGA-1][chipID]);
		printf("\n");
		printf("Memory:     %dk\n",memory);
		printf("Video DAC:  %s\n",dacNames[_getDACType()]);
		printf("\n");
		printf("Separate read/write banks: %s\n", twoBanks ? "Yes" : "No");
		printf("Extended page flipping:    %s\n", pageFlipping ? "Yes" : "No");
		printf("\n");
		printf("Which video mode to test:\n\n");

		i = maxmenu = 0;
		while (modeList[i] != -1) {
			/* Filter out the 16 color video modes */

			switch (modeList[i]) {
				case grSVGA_1280x1024x16:
					if (driver == grSVGA_S3)
						/* This video mode is in a packed 16 color format only
						 * on the S3, and is not compatible with this test
						 * program.
						 */
						break;
				case grEGA_320x200x16:
				case grEGA_640x200x16:
				case grEGA_640x350x16:
				case grVGA_640x480x16:
				case grSVGA_800x600x16:
				case grSVGA_1024x768x16:
					printf("    [%2d] - %s (%d page)\n",maxmenu,
						modeNames[modeList[i]],
						MGL_availablePages(driver,modeList[i]));
					menu[maxmenu++] = modeList[i];
					break;
				}
			i++;
			}
		printf("    [ Q] - Quit\n\n");
		printf("Choice: ");

		gets(buf);
		if (buf[0] == 'q' || buf[0] == 'Q')
			break;

		choice = atoi(buf);
		if (0 <= choice && choice < maxmenu) {
			if (initSuperVGA(driver,chipID,mode = menu[choice],memory) == -1) {
				printf("ERROR: Invalid video mode for driver!\n");
				printf("       Please report this discrepancy...\n");
				exit(1);
				}
			if (!setSuperVGAMode()) {
				printf("\n");
				printf("ERROR: Video mode did not set correctly!\n");
				printf("       Please report this discrepancy...\n");
				printf("\nPress any key to continue...\n");
				getch();
				}
			else {
				getVideoInfo(&maxx,&maxy,&bytesperline,&maxpage);
				moireTest();
				readWriteTest();
				pageFlipTest();
				testingComplete();
				restoreMode();
				}
			}
		}
}
